@@ -353,5 +353,22 @@ jomo2com.MCMCchain <-
     dimnames(upostall)[2]<-list(colnamcovu)
     dimnames(Yimp2)[2] <- list(cnamycomp)
     dimnames(Y2imp2)[2] <- list(cnamy2comp)
+    betapostmean<-data.frame(apply(betapost, c(1,2), mean))
+    beta2postmean<-data.frame(apply(beta2post, c(1,2), mean))
+    upostmean<-data.frame(apply(upostall, c(1,2), mean))
+    omegapostmean<-data.frame(apply(omegapost, c(1,2), mean))
+    covupostmean<-data.frame(apply(covupost, c(1,2), mean))
+    if (output==1) {
+      cat("The posterior mean of the fixed effects estimates is:\n")
+      print(t(betapostmean))
+      cat("\nThe posterior mean of the level 2 fixed effects estimates is:\n")
+      print(t(beta2postmean))
+      cat("\nThe posterior mean of the random effects estimates is:\n")
+      print(upostmean)
+      cat("\nThe posterior mean of the level 1 covariance matrix is:\n")
+      print(omegapostmean)
+      cat("\nThe posterior mean of the level 2 covariance matrix is:\n")
+      print(covupostmean)
+    }
     return(list("finimp"=imp,"collectbeta"=betapost,"collect.l2.beta"=beta2post,"collectomega"=omegapost,"collectu"=upostall, "collectcovu"=covupost, "finimp.latnorm" = Yimp2, "l2.finimp.latnorm" = Y2imp2))
   }
