@@ -11,7 +11,7 @@
 
 SEXP MCMCjomo1glmerbin(SEXP Ysub, SEXP Ysubimp, SEXP Ysubcat, SEXP submod, SEXP ordersub, SEXP submodran, SEXP Y, SEXP Yimp, SEXP Yimp2, SEXP Yimpcat, SEXP X, SEXP Z, SEXP clus, SEXP betaY, SEXP betaYpost, SEXP beta, SEXP u, SEXP uY, SEXP betapost, SEXP upost, SEXP uYpost, SEXP varY, SEXP varYpost, SEXP omega, SEXP omegapost, SEXP covuY, SEXP covuYpost, SEXP covu, SEXP covupost, SEXP nstep, SEXP varYprior, SEXP covuYprior, SEXP Sp, SEXP Sup, SEXP Y_numcat, SEXP num_con, SEXP flagrng){
 int indic=0,i,j,k, IY,JY, IX, JX, Io, Jo, Ib, Jb, ns, nmiss=0,t, countm=0, counto=0,countoo=0, jj, tt, kk, ncon,ncat, pos,flag=0,nmaxx,h=0;
-int Iu, Ju, IZ, JZ, nj,c,fl, currncat, Is,  Il=0, JXm, Ir=0,Jr, JZm, Jum, accratio=0, totprop=0;
+int Iu, Ju, IZ, JZ, nj,c,fl, currncat, Is,  Il=0, JXm, Ir=0,Jr, JZm, Jum, accratio=0, totprop=0, nconnoaux, nconcat, ncatnoaux;
 SEXP RdimY, RdimX, Rdimo, Rdimb, RdimZ, Rdimu, Rdims, Rdimr;
 double *betaX, *Yobs, *Ymiss, *mumiss, *omegadrawmiss, *betamiss, *betaobs, *omegaoo, *omegamo, *omegamm, *invomega, *invomega2, *help, *help2, *help3, *imp, *zi, *yicategorized, *impsub;
 double *sumzy, *incrzz, *incrzy, *mu, *mu2, *newbeta, *newomega, *sumzi, *yi, *invomega3, *help4, *help5, *help6, *missing, *fixomega,meanom,sdom, *resid, logLH, newlogLH,detom, *residsub;
@@ -79,12 +79,17 @@ nstep=PROTECT(coerceVector(nstep,INTSXP));
 ns=INTEGER(nstep)[0];
 num_con=PROTECT(coerceVector(num_con,INTSXP));
 ncon=INTEGER(num_con)[0];
+nconnoaux=INTEGER(num_con)[1];
+nconcat=INTEGER(num_con)[2];
+ncatnoaux=INTEGER(num_con)[3];
 flagrng=PROTECT(coerceVector(flagrng,INTSXP));
 fl=INTEGER(flagrng)[0];
 if (REAL(Yimpcat)[0]==(-999)) ncat=0;
 else ncat=length(Y_numcat);
 nj=Iu;
 
+
+
 for (i=0;i<XLENGTH(ordersub);i++) {
 	for (j=0;j<INTEGER(ordersub)[i];j++) {
 		maxx=maxx*(INTEGER(submod)[3+4*h]);
@@ -227,17 +232,19 @@ for (i=0;i<IY;i++) {
 for (i=0;i<(IY*Ir);i++) Zsub[i]=1;
 for (i=0;i<IY;i++) {
 	pos=0;
-	for (k=0;k<Ir;k++) {
-		if (INTEGER(submodran)[1+pos*3]==1) {
+	for (k=0;k<Jr;k++) {
+		jj=0;
+		if (INTEGER(submodran)[1+k*3]==1) {
 			Zsub[i+IY*pos]=imp[i+IY*(INTEGER(submodran)[k*3]-1)];
 		}
-		else if (INTEGER(submodran)[1+pos*3]==2) {
-			currncat=(INTEGER(submodran)[2+pos*3]);
-			for (jj=1;jj<currncat;jj++) {
-				Zsub[i+IY*(pos+jj-1)]=(REAL(Yimpcat)[i+IY*(INTEGER(submodran)[k*3]-1)]==jj);
+		else if (INTEGER(submodran)[1+k*3]==2) {
+			currncat=(INTEGER(submodran)[2+k*3]);
+			for (jj=0;jj<currncat;jj++) {
+				Zsub[i+IY*(pos+jj)]=(REAL(Yimpcat)[i+IY*(INTEGER(submodran)[k*3]-1)]==(jj+2));
 			}
-		}
-		pos=pos+1;
+		} 
+		pos=pos+1+jj;
+
 	}
 }
 
@@ -252,7 +259,10 @@ for (i=0;i<ns;i++) {
 			pos=ncon;
 			for (j=0;j<ncat;j++) {
 				currncat=INTEGER(Y_numcat)[j]-1;
+							
+
 				for (t=0;t<IY;t++) {
+
 					if (!ISNAN(REAL(Y)[t+(ncon+j)*IY])) {
 						for (k=0;k<currncat;k++) betaX[k]=0;
 						for (tt=0;tt<JX;tt++) {
@@ -265,7 +275,7 @@ for (i=0;i<ns;i++) {
 								betaX[k]=betaX[k]+REAL(u)[(INTEGER(clus)[t])+nj*(tt+(k+pos)*JZ)]*REAL(Z)[t+tt*IZ];
 							}
 						}
-
+						
 						for (k=0;k<(JY-currncat);k++) help[k]=0;
 						for (tt=0;tt<JX;tt++) {
 							for (k=0;k<pos;k++) {
@@ -283,7 +293,7 @@ for (i=0;i<ns;i++) {
 								help[k-currncat]=help[k-currncat]+REAL(u)[(INTEGER(clus)[t])+nj*(tt+k*JZ)]*REAL(Z)[t+tt*IZ];
 							}
 						}
-
+						
 						for (k=0;k<pos;k++) {
 							help[k]=imp[t+k*IY]-help[k];
 						}
@@ -399,6 +409,7 @@ for (i=0;i<ns;i++) {
 			REAL(betapost)[j+Ib*t+i*Ib*Jb]=REAL(beta)[j+Ib*t];
 			}
 		}	
+	
 	//Updating random effects
 		
 	r8mat_pofac(JY*JZ,REAL(covu), help5,1);
@@ -448,10 +459,11 @@ for (i=0;i<ns;i++) {
 		
 	}
 	for (j=0;j<Iu;j++) {
-					for (t=0;t<Ju;t++) {
+		for (t=0;t<Ju;t++) {
 			REAL(upost)[j+Iu*t+i*Iu*Ju]=REAL(u)[j+Iu*t];
 			}
-		}	
+		}
+	
 	//Updating level 2 covariance matrix
 
 		for (j=0;j<Ju*Ju;j++) mu3[j]=0;
@@ -477,6 +489,7 @@ for (i=0;i<ns;i++) {
 			REAL(covupost)[j+Ju*t+i*Ju*Ju]=REAL(covu)[j+Ju*t];
 			}
 		}
+
 	//Updating residuals
 
 	for (t=0;t<IY;t++) {
@@ -508,6 +521,11 @@ for (i=0;i<ns;i++) {
 		r8mat_poinv(JY, help,invomega);
 		for (jj=1;jj<Io;jj++) for (tt=0;tt<jj;tt++) invomega[jj+Io*tt]=invomega[tt+Io*jj];
 		for(k=0;k<Io;k++)  for(j=0;j<Io;j++)  REAL(omega)[k+Io*j]=invomega[k+Io*j];
+		for (j=0;j<JY;j++) {
+			for (t=0;t<JY;t++) {
+				REAL(omegapost)[j+JY*t+i*JY*JY]=REAL(omega)[j+JY*t];
+				}
+			}
 	}
 	else {
 		flag=0;
@@ -566,12 +584,13 @@ for (i=0;i<ns;i++) {
 				}
 			}
 		}
-	}
-	for (j=0;j<Io;j++) {
-		for (t=0;t<Jo;t++) {
-			REAL(omegapost)[j+Io*t+i*Io*Jo]=REAL(omega)[j+Io*t];
+		for (j=0;j<JY;j++) {
+			for (t=0;t<JY;t++) {
+				REAL(omegapost)[j+JY*t+i*JY*JY]=REAL(omega)[j+JY*t];
+				}
 			}
-		}
+
+	}
 		
 
 	//imputing missing values compatibly with substantive model
@@ -610,9 +629,8 @@ for (i=0;i<ns;i++) {
 			mu2[0]=0;
 			for (t=0;t<Il;t++) mu2[0]=mu2[0]+REAL(betaY)[t]*Xsub[j+IY*t];
 			for (t=0;t<Ir;t++) mu2[0]=mu2[0]+REAL(uY)[(INTEGER(clus)[j])+nj*t]*Zsub[j+IY*t];
-			if (REAL(Ysubcat)[j]==0) mu2[0]=-mu2[0];			
+			if (REAL(Ysubcat)[j]==0) mu2[0]=-mu2[0];
 			logLH=log(normal_cdf(mu2[0]))-help6[0]/2;
-			
 		}
 		while (nmiss>0) {
 			if (ISNAN(REAL(Yimp)[j+k*IY])) {
@@ -644,96 +662,113 @@ for (i=0;i<ns;i++) {
 				mumiss[0]=mumiss[0]+betamiss[0];
 				r8mat_mm_new(1,(JY-1),1,omegamo,help3,omegadrawmiss);
 				omegadrawmiss[0]=omegamm[0]-omegadrawmiss[0];
-				Ymiss[0]=r8_normal_sample(yi[k],sqrt(omegamm[0]),0);
 				
-				mu2[0]=0;
-				for (t=0;t<Il;t++) mu2[0]=mu2[0]+REAL(betaY)[t]*Xsub[j+IY*t];
-				for (t=0;t<Ir;t++) mu2[0]=mu2[0]+REAL(uY)[(INTEGER(clus)[j])+nj*t]*Zsub[j+IY*t];
-				if (REAL(Ysubcat)[j]==0) mu2[0]=-mu2[0];			
-				logLH=log(normal_cdf(mu2[0]))-0.5*(pow(yi[k]-mumiss[0],2))/omegadrawmiss[0];
+				if ((k<nconnoaux)||((k>=ncon)&(k<nconcat))) {
+					Ymiss[0]=r8_normal_sample(yi[k],sqrt(omegamm[0]/10),0);
+					mu2[0]=0;
+					for (t=0;t<Il;t++) mu2[0]=mu2[0]+REAL(betaY)[t]*Xsub[j+IY*t];
+					for (t=0;t<Ir;t++) mu2[0]=mu2[0]+REAL(uY)[(INTEGER(clus)[j])+nj*t]*Zsub[j+IY*t];
 
-				// Controllare se accettabile
-				yi[k]=Ymiss[0];								
-				if (ncat>0) {
-					h=0;
-					for (jj=0;jj<ncat;jj++) {
-						maxx=yi[(ncon+h)];
-						nmaxx=0;
-						for (kk=1;kk<(INTEGER(Y_numcat)[jj]-1);kk++) {
-							if (yi[(ncon+h+kk)]>maxx) {
-								maxx=yi[(ncon+h+kk)];
-								nmaxx=kk;
-							}
-						}			
-						if (maxx>0) yicategorized[jj]=nmaxx;
-						else yicategorized[jj]=INTEGER(Y_numcat)[j]-1;
-						h=h+INTEGER(Y_numcat)[jj]-1;
-					}
-				}
-					
-					//Update Xsubprop
-				h=0;
-				indic=0;
-				for (t=0;t<Il;t++) Xsubprop[t]=1;
-
-				for (jj=0;jj<XLENGTH(ordersub);jj++) {
-					pos=1;
-					currncat=1;
-					for (t=0;t<INTEGER(ordersub)[jj];t++) {
-						pos=pos*(INTEGER(submod)[3+(h+t)*4]);
-					}
-					for (t=0;t<INTEGER(ordersub)[jj];t++) {
-						if (INTEGER(submod)[1+h*4]==1) {
-							for (tt=0;tt<pos;tt++) {
-								Xsubprop[(1+tt+indic)]=Xsubprop[(1+tt+indic)]*pow(yi[(INTEGER(submod)[h*Is]-1)],INTEGER(submod)[2+h*Is]);
-
-							}	
-						} else {
-							for (tt=0;tt<pos;tt++) {
-								kk=(tt*currncat)%INTEGER(submod)[3+h*4]+1;
-								Xsubprop[(1+tt+indic)]=Xsubprop[(1+tt+indic)]*(yicategorized[(INTEGER(submod)[h*Is]-1)]==kk);
-							}
-						}	
-						currncat=currncat*INTEGER(submod)[3+h*4];
-						h=h+1;
-					}
-					currncat=1;
-					indic=indic+pos;
-				}
-				
-				   // Update Zsubprop
-
-				for (t=0;t<Ir;t++) Zsubprop[t]=1;
-				
-				for (t=0;t<Ir;t++) {
-					if (INTEGER(submodran)[1+t*3]==1) {
-						Zsubprop[t]=yi[(INTEGER(submodran)[t*3]-1)];
-					} else if (INTEGER(submodran)[1+t*3]==2) {
-						currncat=(INTEGER(submodran)[2+t*3]);
-						for (jj=1;jj<currncat;jj++) {
-							Zsubprop[t+jj-1]=(yicategorized[(INTEGER(submodran)[t*3]-1)]==jj);
+					if (REAL(Ysubcat)[j]==0) mu2[0]=-mu2[0];
+					logLH=log(normal_cdf(mu2[0]))-help6[0]/2;
+					// Controllare se accettabile
+					help4[k]=help4[k]+Ymiss[0]-yi[k];
+					r8mat_mm_new(1,JY,JY,help4,invomega,help5);
+					r8mat_mmt_new(1,JY,1,help5,help4,help);
+					yi[k]=Ymiss[0];								
+					if ((ncatnoaux>0)) {
+						h=0;
+						for (jj=0;jj<(ncatnoaux);jj++) {
+							maxx=yi[(ncon+h)];
+							nmaxx=0;
+							if (INTEGER(Y_numcat)[jj]>2) {
+								for (kk=1;kk<(INTEGER(Y_numcat)[jj]-1);kk++) {
+									if (yi[(ncon+h+kk)]>maxx) {
+										maxx=yi[(ncon+h+kk)];
+										nmaxx=kk;
+									}
+								}
+							}			
+							if (maxx>0) yicategorized[jj]=nmaxx;
+							else yicategorized[jj]=INTEGER(Y_numcat)[jj]-1;
+							h=h+INTEGER(Y_numcat)[jj]-1;
 						}
 					}
-				pos=pos+1;
-				}				
-				
-				mu2[0]=0;
-				for (t=0;t<Il;t++) mu2[0]=mu2[0]+REAL(betaY)[t]*Xsubprop[t];
-				for (t=0;t<Ir;t++) mu2[0]=mu2[0]+REAL(uY)[(INTEGER(clus)[j])+nj*t]*Zsubprop[t];
-				if (REAL(Ysubcat)[j]==0) mu2[0]=-mu2[0];			
-				newlogLH=log(normal_cdf(mu2[0]))-0.5*(pow(Ymiss[0]-mumiss[0],2))/omegadrawmiss[0];
-				
-				if ((( double ) unif_rand ( ) )<exp(newlogLH-logLH)) {	
+					
+						//Update Xsubprop
+					h=0;
+					indic=0;
+					for (t=0;t<Il;t++) Xsubprop[t]=1;
 
-					imp[j+k*IY]=Ymiss[0];
-					for (t=0;t<Il;t++) Xsub[j+t*IY]=Xsubprop[t];
-					for (t=0;t<Ir;t++) Zsub[j+t*IY]=Zsubprop[t];
-					logLH=newlogLH;
-					 accratio=accratio+1;
+					for (jj=0;jj<XLENGTH(ordersub);jj++) {
+						pos=1;
+						currncat=1;
+						for (t=0;t<INTEGER(ordersub)[jj];t++) {
+							pos=pos*(INTEGER(submod)[3+(h+t)*4]);
+						}
+						for (t=0;t<INTEGER(ordersub)[jj];t++) {
+							if (INTEGER(submod)[1+h*4]==1) {
+								for (tt=0;tt<pos;tt++) {
+									Xsubprop[(1+tt+indic)]=Xsubprop[(1+tt+indic)]*pow(yi[(INTEGER(submod)[h*Is]-1)],INTEGER(submod)[2+h*Is]);
+
+								}	
+							} else {
+								for (tt=0;tt<pos;tt++) {
+									kk=(tt*currncat)%INTEGER(submod)[3+h*4]+1;
+									Xsubprop[(1+tt+indic)]=Xsubprop[(1+tt+indic)]*(yicategorized[(INTEGER(submod)[h*Is]-1)]==kk);
+								}
+							}	
+							currncat=currncat*INTEGER(submod)[3+h*4];
+							h=h+1;
+						}
+						currncat=1;
+						indic=indic+pos;
+					}
+				
+										// Update Zsubprop
+
+					for (t=0;t<Ir;t++) Zsubprop[t]=1;
+					pos=0;
+					for (t=0;t<Jr;t++) {
+						jj=0;
+						if (INTEGER(submodran)[1+t*3]==1) {
+							Zsubprop[t]=yi[(INTEGER(submodran)[t*3]-1)];
+						} else if (INTEGER(submodran)[1+t*3]==2) {
+							currncat=(INTEGER(submodran)[2+t*3]);
+							for (jj=0;jj<currncat;jj++) {
+								Zsubprop[pos+jj]=(yicategorized[(INTEGER(submodran)[t*3]-1)]==(jj+1));
+							}
+						} 
+						pos=pos+1+jj;
+					}					
+			
+ 			
+ 					mu2[0]=0;
+					for (t=0;t<Il;t++) mu2[0]=mu2[0]+REAL(betaY)[t]*Xsubprop[t];
+					for (t=0;t<Ir;t++) mu2[0]=mu2[0]+REAL(uY)[(INTEGER(clus)[j])+nj*t]*Zsubprop[t];
+					
+					if (REAL(Ysubcat)[j]==0) mu2[0]=-mu2[0];	
+					newlogLH=log(normal_cdf(mu2[0]))-help[0]/2;
+					if ((( double ) unif_rand ( ) )<exp(newlogLH-logLH)) {	
+
+						imp[j+k*IY]=Ymiss[0];
+						help6[0]=help[0];
+						for (t=0;t<Il;t++) Xsub[j+t*IY]=Xsubprop[t];
+						for (t=0;t<Ir;t++) Zsub[j+t*IY]=Zsubprop[t];
+						logLH=newlogLH;
+						accratio=accratio+1;
+					} else {
+						help4[k]=help4[k]-Ymiss[0]+imp[j+k*IY];
+						yi[k]=imp[j+k*IY];
+					}
+					totprop=totprop+1;				
 				} else {
-					yi[k]=imp[j+k*IY];
+					Ymiss[0]=r8_normal_sample(mumiss[0],sqrt(omegadrawmiss[0]),0);
+					imp[j+k*IY]=Ymiss[0];
+					help4[k]=help4[k]+Ymiss[0]-yi[k];
+					r8mat_mm_new(1,JY,JY,help4,invomega,help5);
+					r8mat_mmt_new(1,JY,1,help5,help4,help6);
 				}
-				totprop=totprop+1;				
 				nmiss--;
 				counto=0;
 				countoo=0;
@@ -745,25 +780,25 @@ for (i=0;i<ns;i++) {
 	}
 
 	// Rejection sampling for latent normal outcome
-
-	for (t=0;t<IY;t++) {
-		if (!ISNAN(REAL(Ysub)[t])) {
-			kk=0;
-			flag=0;
-			mu2[0]=0;
-			for (k=0;k<Il;k++) mu2[0]=mu2[0]+REAL(betaY)[k]*Xsub[t+k*IX];
-			for (k=0;k<Ir;k++) mu2[0]=mu2[0]+REAL(uY)[(INTEGER(clus)[t])+nj*k]*Zsub[t+k*IX];
-			while (flag==0&&kk<10000) {
-				yi[0]=r8_normal_sample(mu2[0],sqrt(REAL(varY)[0]),0);
-				if ((REAL(Ysub)[t]==1&&yi[0]>0)||(REAL(Ysub)[t]==0&&yi[0]<0)) {
-					impsub[t]=yi[0];
-					flag=1;
-				} else {
-					kk++;
+	
+		for (t=0;t<IY;t++) {
+			if (!ISNAN(REAL(Ysub)[t])) {
+				kk=0;
+				flag=0;
+				mu2[0]=0;
+				for (k=0;k<Il;k++) mu2[0]=mu2[0]+REAL(betaY)[k]*Xsub[t+k*IX];
+				for (k=0;k<Ir;k++) mu2[0]=mu2[0]+REAL(uY)[(INTEGER(clus)[t])+nj*k]*Zsub[t+k*IX];
+				while (flag==0&&kk<10000) {
+					yi[0]=r8_normal_sample(mu2[0],sqrt(REAL(varY)[0]),0);
+					if ((REAL(Ysub)[t]==1&&yi[0]>0)||(REAL(Ysub)[t]==0&&yi[0]<0)) {
+						impsub[t]=yi[0];
+						flag=1;
+					} else {
+						kk++;
+					}
 				}
 			}
 		}
-	}
 	
 	// Update beta of substantive model
 	
@@ -790,7 +825,7 @@ for (i=0;i<ns;i++) {
 	r8mat_mm_new(Il,Il,1,invomega2,sumxy,mu);
 	r8mat_pofac(Il,invomega2,help3,5);
 	r8vec_multinormal_sample(Il, mu,help3, REAL(betaY),newbeta,0);	
-	for (t=0;t<Il;t++) REAL(betaYpost)[t+i*Il]=REAL(betaY)[t];
+	for (j=0;j<Il;j++) REAL(betaYpost)[j+i*Il]=REAL(betaY)[j];
 
 	// Update random effects of substantive model
 	r8mat_pofac(Ir,REAL(covuY), help5,1);
@@ -831,9 +866,9 @@ for (i=0;i<ns;i++) {
 	}
 	for (j=0;j<nj;j++) {
 		for (t=0;t<Ir;t++) {
- 					REAL(uYpost)[j+nj*t+i*nj*Ir]=REAL(uY)[j+nj*t];
+			REAL(uYpost)[j+nj*t+i*nj*Ir]=REAL(uY)[j+nj*t];
 			}
-		}
+		}	
 	//Updating level 2 covariance matrix of substantive model 
 	
 	for (j=0;j<Ir*Ir;j++) mu3[j]=0;
@@ -853,7 +888,7 @@ for (i=0;i<ns;i++) {
 	r8mat_poinv(Ir, help4,invomega3);
 	for (jj=1;jj<Ir;jj++) for (tt=0;tt<jj;tt++) invomega3[jj+Ir*tt]=invomega3[tt+Ir*jj];
 	for(k=0;k<Ir*Ir;k++) REAL(covuY)[k]=invomega3[k];	
-	for (j=0;j<Ir;j++) {
+for (j=0;j<Ir;j++) {
 		for (t=0;t<Ir;t++) {
 			REAL(covuYpost)[j+Ir*t+i*Ir*Ir]=REAL(covuY)[j+Ir*t];
 			}
@@ -870,10 +905,10 @@ for (i=0;i<ns;i++) {
 			}
 	}
 	
-		//Not updating omega (fixed to 1)
-
-	REAL(varYpost)[i]=REAL(varY)[0];
+	//Not updating omega (fixed to 1)
 	
+	REAL(varYpost)[i]=REAL(varY)[0];
+
 		// Imputing missing outcomes
 	for (t=0;t<IY;t++) {
 		if (ISNAN(REAL(Ysub)[t])) {
@@ -885,13 +920,12 @@ for (i=0;i<ns;i++) {
 			REAL(Ysubcat)[t]=(impsub[t]>0);
 		}
 	}
-	
+		if (i==100) Rprintf("beta=%f betaY=%f\n", REAL(beta)[0], REAL(betaY)[0]);
+
 	if ((i+1)%fl==0) Rprintf(".");
 }
-if (fl==1) Rprintf("\n");
-
-if (((double)accratio/((double)totprop))<0.3) Rprintf("Warning: acceptance ratio = %f. This might be a sign that the chain did not mix well. \n" , ((double)accratio/((double)totprop)));
-
+	if (fl==1) Rprintf("\n");
+if (((double)accratio/((double)totprop))<0.15) Rprintf("Warning: acceptance ratio = %f. This might be a sign that the chain did not mix well. \n" , ((double)accratio/((double)totprop)));
 for(i=0;i<IY;i++)  {
 	for(j=0;j<JY;j++)  {
 		REAL(Yimp2)[i+IY*j]=imp[i+IY*j];
