@@ -10,8 +10,8 @@
 #include<Rmath.h>
 
 SEXP MCMCjomo2glmerbinhr(SEXP Ysub, SEXP Ysubimp, SEXP Ysubcat, SEXP submod, SEXP ordersub, SEXP submodran, SEXP Y, SEXP Yimp, SEXP Yimp2, SEXP Yimpcat, SEXP Y2, SEXP Y2imp, SEXP Y2imp2, SEXP Y2impcat, SEXP X, SEXP X2, SEXP Z, SEXP clus, SEXP betaY, SEXP betaYpost, SEXP beta, SEXP beta2, SEXP u, SEXP uY, SEXP betapost, SEXP upost, SEXP uYpost, SEXP beta2post, SEXP varY, SEXP varYpost, SEXP omega, SEXP omegapost, SEXP covuY, SEXP covuYpost, SEXP covu, SEXP covupost, SEXP nstep, SEXP varYprior, SEXP covuYprior, SEXP Sp, SEXP Sup, SEXP Y_numcat, SEXP Y2_numcat, SEXP num_con, SEXP num_con2,SEXP a_start, SEXP a_prior, SEXP flagrng){
-int indic=0,i,j,k, IY,JY, IX, JX, Io, Jo, Ib, Jb, ns, nmiss=0,t, countm=0, counto=0, countoo=0, jj, tt, kk, ncon,ncat, pos,flag=0,nmaxx,h=0;
-int Iu, Ju, IZ, JZ, nj,c,fl, currncat, IY2, JY2, IX2,JX2,Ib2, Jb2, ncon2, ncat2, JYm, JXm, Is, Il=0, Ir=0,Jr, JZm, Jum, accratio=0, totprop=0, accratio2=0, totprop2=0;
+int indic=0,i,j,k, IY,JY, IX, JX, Io, Jo, Ib, Jb, ns, nmiss=0,t, countm=0, counto=0, countoo=0, jj, tt, kk, ncon,ncat, pos,flag=0,nmaxx,h=0, nconnoaux, nconcat, ncatnoaux;
+int Iu, Ju, IZ, JZ, nj,c,fl, currncat, IY2, JY2, IX2,JX2,Ib2, Jb2, ncon2, ncat2, JYm, JXm, Is, Il=0, Ir=0,Jr, JZm, Jum, accratio=0, totprop=0, accratio2=0, totprop2=0, nconnoaux2, nconcat2, ncatnoaux2;
 SEXP RdimY, RdimX, Rdimo, Rdimb, RdimZ, Rdimu, RdimY2, RdimX2, Rdimb2, Rdims, Rdimr;
 double *betaX, *Yobs, *Ymiss, *mumiss, *omegadrawmiss, *betamiss, *betaobs, *omegaoo, *omegamo, *omegamm, *invomega, *invomega2, *help, *help2, *help3, *imp,*imp2, *zi, *yicategorized;
 double *sumzy, *incrzz, *incrzy, *mu, *mu2, *newbeta, *newomega, *sumzi, *yi, *invomega3, *help4, *help5, *help6, *missing, *fixomega,meanom,sdom, *resid, logLH, newlogLH,detom;
@@ -98,8 +98,14 @@ nstep=PROTECT(coerceVector(nstep,INTSXP));
 ns=INTEGER(nstep)[0];
 num_con=PROTECT(coerceVector(num_con,INTSXP));
 ncon=INTEGER(num_con)[0];
+nconnoaux=INTEGER(num_con)[1];		
+nconcat=INTEGER(num_con)[2];		
+ncatnoaux=INTEGER(num_con)[3];
 num_con2=PROTECT(coerceVector(num_con2,INTSXP));
 ncon2=INTEGER(num_con2)[0];
+nconnoaux2=INTEGER(num_con2)[1];		
+nconcat2=INTEGER(num_con2)[2];		
+ncatnoaux2=INTEGER(num_con2)[3];
 flagrng=PROTECT(coerceVector(flagrng,INTSXP));
 fl=INTEGER(flagrng)[0];
 if (REAL(Yimpcat)[0]==(-999)) ncat=0;
@@ -246,7 +252,6 @@ for (j=0; j<Iu; j++) {
 		}
 	}
 }
-
 for (i=0;i<JY*JY;i++) fixomega[i]=0;
 for (i=0;i<Ju*Ju;i++) fixomega2[i]=0;
 pos=ncon;
@@ -333,17 +338,19 @@ for (i=0;i<IY;i++) {
 for (i=0;i<(IY*Ir);i++) Zsub[i]=1;
 for (i=0;i<IY;i++) {
 	pos=0;
-	for (k=0;k<Ir;k++) {
-		if (INTEGER(submodran)[1+pos*3]==1) {
+	for (k=0;k<Jr;k++) {
+		jj=0;
+		if (INTEGER(submodran)[1+k*3]==1) {
 			Zsub[i+IY*pos]=imp[i+IY*(INTEGER(submodran)[k*3]-1)];
 		}
-		else if (INTEGER(submodran)[1+pos*3]==2) {
-			currncat=(INTEGER(submodran)[2+pos*3]);
-			for (jj=1;jj<currncat;jj++) {
-				Zsub[i+IY*(pos+jj-1)]=(REAL(Yimpcat)[i+IY*(INTEGER(submodran)[k*3]-1)]==jj);
+		else if (INTEGER(submodran)[1+k*3]==2) {
+			currncat=(INTEGER(submodran)[2+k*3]);
+			for (jj=0;jj<currncat;jj++) {
+				Zsub[i+IY*(pos+jj)]=(REAL(Yimpcat)[i+IY*(INTEGER(submodran)[k*3]-1)]==(jj+2));
 			}
-		}
-		pos=pos+1;
+		} 
+		pos=pos+1+jj;
+
 	}
 }
 
@@ -575,7 +582,6 @@ for (i=0;i<ns;i++) {
 
 		}
 
-		
 	//Updating beta
 
 	for (j=0;j<JY*JY*JX*JX;j++) sumxi[j]=0;	
@@ -655,7 +661,7 @@ for (i=0;i<ns;i++) {
 					for (t=0;t<Jb2;t++) {
 			REAL(beta2post)[j+Ib2*t+i*Ib2*Jb2]=REAL(beta2)[j+Ib2*t];
 			}
-		}	
+		}		
 	// Calculating level 2 residuals 
 	
 	for (j=0;j<Iu;j++) {
@@ -733,7 +739,7 @@ for (i=0;i<ns;i++) {
 					for (t=0;t<Ju;t++) {
 			REAL(upost)[j+Iu*t+i*Iu*Ju]=REAL(u)[j+Iu*t];
 			}
-		}	
+		}		
 	//Updating level 2 covariance matrix
 
 	if (ncat2==0) {
@@ -754,7 +760,13 @@ for (i=0;i<ns;i++) {
 		r8mat_poinv(Ju, help4,invomega3);
 		for (jj=1;jj<(Ju);jj++) for (tt=0;tt<jj;tt++) invomega3[jj+(Ju)*tt]=invomega3[tt+(Ju)*jj];
 		for(k=0;k<(Ju);k++)  for(j=0;j<(Ju);j++)  REAL(covu)[j+(Ju)*k]=invomega3[j+(Ju)*k];
-	}
+	
+	for (j=0;j<Ju;j++) {
+		for (t=0;t<Ju;t++) {
+			REAL(covupost)[j+Ju*t+i*Ju*Ju]=REAL(covu)[j+Ju*t];
+			}					
+		}
+		}
 	else {
 			flag=0;
 			r8mat_pofac(Ju,REAL(covu),help,12);
@@ -812,13 +824,15 @@ for (i=0;i<ns;i++) {
 						flag=0;
 					}
 				}
-			}				
-	}
+			}
+						
 	for (j=0;j<Ju;j++) {
 		for (t=0;t<Ju;t++) {
 			REAL(covupost)[j+Ju*t+i*Ju*Ju]=REAL(covu)[j+Ju*t];
 			}					
 		}
+		}
+
 	//Updating residuals
 
 	for (t=0;t<IY;t++) {
@@ -951,11 +965,12 @@ for (i=0;i<ns;i++) {
 			}
 		}
 	}
-	for (j=0;j<Io;j++) {
+for (j=0;j<Io;j++) {
 		for (t=0;t<Jo;t++) {
 			REAL(omegapost)[j+Io*t+i*Io*Jo]=REAL(omega)[j+Io*t];
 			}
 		}
+		
 	for (c=0;c<nj;c++) {
 		for (j=0;j<JY;j++) {
 			for (t=0;t<JY;t++) newomega[j+t*JY]=REAL(omega)[(c*JY+j)+t*(JY*nj)];
@@ -1005,154 +1020,176 @@ for (i=0;i<ns;i++) {
 			mu2[0]=0;
 			for (t=0;t<Il;t++) mu2[0]=mu2[0]+REAL(betaY)[t]*Xsub[j+IY*t];
 			for (t=0;t<Ir;t++) mu2[0]=mu2[0]+REAL(uY)[(INTEGER(clus)[j])+nj*t]*Zsub[j+IY*t];
-			if (REAL(Ysubcat)[j]==0) mu2[0]=-mu2[0];			
+			if (REAL(Ysubcat)[j]==0) mu2[0]=-mu2[0];
 			logLH=log(normal_cdf(mu2[0]))-help6[0]/2;
-		}
-		while (nmiss>0) {
-			if (ISNAN(REAL(Yimp)[j+k*IY])) {
-				betamiss[0]=betaX[k];
-				omegamm[0]=REAL(omega)[INTEGER(clus)[j]*JY+k+k*Io];
-				for (t=0;t<JY;t++) {
-					if (t!=k) {
-						betaobs[counto]=betaX[t];
-						omegamo[counto]=REAL(omega)[INTEGER(clus)[j]*JY+k+t*Io];
-						Yobs[counto]=imp[j+t*IY];
-						for (tt=0;tt<JY;tt++) {
-							if (tt!=k) {
-								omegaoo[countoo]=REAL(omega)[INTEGER(clus)[j]*JY+tt+t*Io];
-								countoo++;
+
+			}
+			while (nmiss>0) {
+				if (ISNAN(REAL(Yimp)[j+k*IY])) {
+					betamiss[0]=betaX[k];
+					omegamm[0]=REAL(omega)[INTEGER(clus)[j]*JY+k+k*Io];
+					for (t=0;t<JY;t++) {
+						if (t!=k) {
+							betaobs[counto]=betaX[t];
+							omegamo[counto]=REAL(omega)[INTEGER(clus)[j]*JY+k+t*Io];
+							Yobs[counto]=imp[j+t*IY];
+							for (tt=0;tt<JY;tt++) {
+								if (tt!=k) {
+									omegaoo[countoo]=REAL(omega)[INTEGER(clus)[j]*JY+tt+t*Io];
+									countoo++;
+								}
 							}
+							counto++;
 						}
-						counto++;
 					}
-				}
 
-				r8mat_pofac((JY-1),omegaoo,help2,14);
-				r8mat_poinv((JY-1),help2,invomega3);
-				for (jj=1;jj<JY-1;jj++) for (tt=0;tt<jj;tt++) invomega3[jj+(JY-1)*tt]=invomega3[tt+(JY-1)*jj];
-				r8mat_mmt_new((JY-1),(JY-1),1,invomega3,omegamo,help3);
-				r8mat_divide((JY-1),1,-1,betaobs);
-				r8mat_add((JY-1),1,betaobs,Yobs);
-				r8mat_mtm_new(1,(JY-1),1,Yobs,help3,mumiss);
+					r8mat_pofac((JY-1),omegaoo,help2,14);
+					r8mat_poinv((JY-1),help2,invomega3);
+					for (jj=1;jj<JY-1;jj++) for (tt=0;tt<jj;tt++) invomega3[jj+(JY-1)*tt]=invomega3[tt+(JY-1)*jj];
+					r8mat_mmt_new((JY-1),(JY-1),1,invomega3,omegamo,help3);
+					r8mat_divide((JY-1),1,-1,betaobs);
+					r8mat_add((JY-1),1,betaobs,Yobs);
+					r8mat_mtm_new(1,(JY-1),1,Yobs,help3,mumiss);
 
-				mumiss[0]=mumiss[0]+betamiss[0];
-				r8mat_mm_new(1,(JY-1),1,omegamo,help3,omegadrawmiss);
-				omegadrawmiss[0]=omegamm[0]-omegadrawmiss[0];
-				Ymiss[0]=r8_normal_sample(yi[k],sqrt(omegamm[0]),0);
+					mumiss[0]=mumiss[0]+betamiss[0];
+					r8mat_mm_new(1,(JY-1),1,omegamo,help3,omegadrawmiss);
+					omegadrawmiss[0]=omegamm[0]-omegadrawmiss[0];
 				
-				mu2[0]=0;
-				for (t=0;t<Il;t++) mu2[0]=mu2[0]+REAL(betaY)[t]*Xsub[j+IY*t];
-				for (t=0;t<Ir;t++) mu2[0]=mu2[0]+REAL(uY)[(INTEGER(clus)[j])+nj*t]*Zsub[j+IY*t];
-				if (REAL(Ysubcat)[j]==0) mu2[0]=-mu2[0];			
-				logLH=log(normal_cdf(mu2[0]))-0.5*(pow(yi[k]-mumiss[0],2))/omegadrawmiss[0];
+					if ((k<nconnoaux)||((k>=ncon)&(k<nconcat))) {
+						Ymiss[0]=r8_normal_sample(yi[k],sqrt(omegamm[0]/10),0);
+						mu2[0]=0;
+						for (t=0;t<Il;t++) mu2[0]=mu2[0]+REAL(betaY)[t]*Xsub[j+IY*t];
+						for (t=0;t<Ir;t++) mu2[0]=mu2[0]+REAL(uY)[(INTEGER(clus)[j])+nj*t]*Zsub[j+IY*t];
+						if (REAL(Ysubcat)[j]==0) mu2[0]=-mu2[0];
+						logLH=log(normal_cdf(mu2[0]))-help6[0]/2;
+
 
 				// Controllare se accettabile
-				yi[k]=Ymiss[0];								
-				if (ncat>0) {
-					h=0;
-					for (jj=0;jj<ncat;jj++) {
-						maxx=yi[(ncon+h)];
-						nmaxx=0;
-						for (kk=1;kk<(INTEGER(Y_numcat)[jj]-1);kk++) {
-							if (yi[(ncon+h+kk)]>maxx) {
-								maxx=yi[(ncon+h+kk)];
-								nmaxx=kk;
+						help4[k]=help4[k]+Ymiss[0]-yi[k];
+						r8mat_mm_new(1,JY,JY,help4,invomega,help5);
+						r8mat_mmt_new(1,JY,1,help5,help4,help);
+						yi[k]=Ymiss[0];								
+						if (ncatnoaux>0) {
+							h=0;
+							for (jj=0;jj<ncatnoaux;jj++) {
+								maxx=yi[(ncon+h)];
+								nmaxx=0;
+								if (INTEGER(Y_numcat)[jj]>2) {
+									for (kk=1;kk<(INTEGER(Y_numcat)[jj]-1);kk++) {
+										if (yi[(ncon+h+kk)]>maxx) {
+											maxx=yi[(ncon+h+kk)];
+											nmaxx=kk;
+										}
+									}
+								}			
+								if (maxx>0) yicategorized[jj]=nmaxx;
+								else yicategorized[jj]=INTEGER(Y_numcat)[jj]-1;
+								h=h+INTEGER(Y_numcat)[jj]-1;
 							}
-						}			
-						if (maxx>0) yicategorized[jj]=nmaxx;
-						else yicategorized[jj]=INTEGER(Y_numcat)[jj]-1;
-						h=h+INTEGER(Y_numcat)[jj]-1;
-					}
-				}
-				if (ncat2>0) {
-					h=0;
-					for (jj=0;jj<ncat2;jj++) {
-						maxx=imp2[INTEGER(clus)[j]+Iu*(ncon2+h)];
-						nmaxx=0;
-						for (kk=1;kk<(INTEGER(Y2_numcat)[jj]-1);kk++) {
-							if (imp2[INTEGER(clus)[j]+Iu*(ncon2+h+kk)]>maxx) {
-								maxx=imp2[INTEGER(clus)[j]+Iu*(ncon2+h+kk)];
-								nmaxx=kk;
+						}
+						if (ncatnoaux2>0) {
+							h=0;
+							for (jj=0;jj<ncatnoaux2;jj++) {
+								maxx=imp2[INTEGER(clus)[j]+Iu*(ncon2+h)];
+								nmaxx=0;
+								if (INTEGER(Y2_numcat)[jj]>2) {
+									for (kk=1;kk<(INTEGER(Y2_numcat)[jj]-1);kk++) {
+										if (imp2[INTEGER(clus)[j]+Iu*(ncon2+h+kk)]>maxx) {
+											maxx=imp2[INTEGER(clus)[j]+Iu*(ncon2+h+kk)];
+											nmaxx=kk;
+										}
+									}
+								}			
+								if (maxx>0) yi2categorized[jj]=nmaxx;
+								else yi2categorized[jj]=INTEGER(Y2_numcat)[jj]-1;
+								h=h+INTEGER(Y2_numcat)[jj]-1;
 							}
-						}			
-						if (maxx>0) yi2categorized[jj]=nmaxx;
-						else yi2categorized[jj]=INTEGER(Y2_numcat)[jj]-1;
-						h=h+INTEGER(Y2_numcat)[jj]-1;
-					}
-				}
+						}
 					
 					//Update Xsubprop
-				h=0;
-				indic=0;
-				for (t=0;t<Il;t++) Xsubprop[t]=1;
+						h=0;
+						indic=0;
+						for (t=0;t<Il;t++) Xsubprop[t]=1;
+	
+						for (jj=0;jj<XLENGTH(ordersub);jj++) {
+							pos=1;
+							currncat=1;
+							for (t=0;t<INTEGER(ordersub)[jj];t++) {
+								pos=pos*(INTEGER(submod)[3+(h+t)*4]);
+							}
+							for (t=0;t<INTEGER(ordersub)[jj];t++) {
+								if (INTEGER(submod)[1+h*4]==1) {
+									for (tt=0;tt<pos;tt++) {
+										Xsubprop[1+tt+indic]=Xsubprop[1+tt+indic]*pow(yi[(INTEGER(submod)[h*Is]-1)],INTEGER(submod)[2+h*Is]);
 
-				for (jj=0;jj<XLENGTH(ordersub);jj++) {
-					pos=1;
-					currncat=1;
-					for (t=0;t<INTEGER(ordersub)[jj];t++) {
-						pos=pos*(INTEGER(submod)[3+(h+t)*4]);
-					}
-					for (t=0;t<INTEGER(ordersub)[jj];t++) {
-						if (INTEGER(submod)[1+h*4]==1) {
-							for (tt=0;tt<pos;tt++) {
-								Xsubprop[1+tt+indic]=Xsubprop[1+tt+indic]*pow(yi[(INTEGER(submod)[h*Is]-1)],INTEGER(submod)[2+h*Is]);
-
-							}	
-						} else if (INTEGER(submod)[1+h*4]==2) {
-							for (tt=0;tt<pos;tt++) {
-								kk=(tt*currncat)%INTEGER(submod)[3+h*4]+1;
-								Xsubprop[1+tt+indic]=Xsubprop[1+tt+indic]*(yicategorized[(INTEGER(submod)[h*Is]-1)]==kk);
-							}
-						} else if (INTEGER(submod)[1+h*4]==3) {
-							for (tt=0;tt<pos;tt++) {
-								Xsubprop[1+tt+indic]=Xsubprop[1+tt+indic]*pow(imp2[INTEGER(clus)[j]+Iu*(INTEGER(submod)[h*Is]-1)],INTEGER(submod)[2+h*Is]);
-							}
-						} else if (INTEGER(submod)[1+h*4]==4) {
-							for (tt=0;tt<pos;tt++) {
-								kk=(tt*currncat)%INTEGER(submod)[3+h*4]+1;
-								Xsubprop[1+tt+indic]=Xsubprop[1+tt+indic]*(yi2categorized[(INTEGER(submod)[h*Is]-1)]==kk);
+									}	
+								} else if (INTEGER(submod)[1+h*4]==2) {
+									for (tt=0;tt<pos;tt++) {
+										kk=(tt*currncat)%INTEGER(submod)[3+h*4]+1;
+										Xsubprop[1+tt+indic]=Xsubprop[1+tt+indic]*(yicategorized[(INTEGER(submod)[h*Is]-1)]==kk);
+									}
+								} else if (INTEGER(submod)[1+h*4]==3) {
+									for (tt=0;tt<pos;tt++) {
+										Xsubprop[1+tt+indic]=Xsubprop[1+tt+indic]*pow(imp2[INTEGER(clus)[j]+Iu*(INTEGER(submod)[h*Is]-1)],INTEGER(submod)[2+h*Is]);
+									}
+								} else if (INTEGER(submod)[1+h*4]==4) {
+									for (tt=0;tt<pos;tt++) {
+										kk=(tt*currncat)%INTEGER(submod)[3+h*4]+1;
+										Xsubprop[1+tt+indic]=Xsubprop[1+tt+indic]*(yi2categorized[(INTEGER(submod)[h*Is]-1)]==kk);
+									}
+								}
+								currncat=currncat*INTEGER(submod)[3+h*4];
+								h=h+1;
 							}
+							currncat=1;
+							indic=indic+pos;
 						}
-						currncat=currncat*INTEGER(submod)[3+h*4];
-						h=h+1;
-					}
-					currncat=1;
-					indic=indic+pos;
-				}
 				
-				   // Update Zsubprop
+								// Update Zsubprop
 
-				for (t=0;t<Ir;t++) Zsubprop[t]=1;
+					for (t=0;t<Ir;t++) Zsubprop[t]=1;
+					pos=0;
+					for (t=0;t<Jr;t++) {
+						jj=0;
+						if (INTEGER(submodran)[1+t*3]==1) {
+							Zsubprop[t]=yi[(INTEGER(submodran)[t*3]-1)];
+						} else if (INTEGER(submodran)[1+t*3]==2) {
+							currncat=(INTEGER(submodran)[2+t*3]);
+							for (jj=0;jj<currncat;jj++) {
+								Zsubprop[pos+jj]=(yicategorized[(INTEGER(submodran)[t*3]-1)]==(jj+1));
+							}
+						} 
+						pos=pos+1+jj;
+					}					
+			
 				
-				for (t=0;t<Ir;t++) {
-					if (INTEGER(submodran)[1+t*3]==1) {
-						Zsubprop[t]=yi[(INTEGER(submodran)[t*3]-1)];
-					} else if (INTEGER(submodran)[1+t*3]==2) {
-						currncat=(INTEGER(submodran)[2+t*3]);
-						for (jj=1;jj<currncat;jj++) {
-							Zsubprop[t+jj-1]=(yicategorized[(INTEGER(submodran)[t*3]-1)]==jj);
-						}
-					}
-				pos=pos+1;
-				}				
-				
-				mu2[0]=0;
-				for (t=0;t<Il;t++) mu2[0]=mu2[0]+REAL(betaY)[t]*Xsubprop[t];
-				for (t=0;t<Ir;t++) mu2[0]=mu2[0]+REAL(uY)[(INTEGER(clus)[j])+nj*t]*Zsubprop[t];
-				if (REAL(Ysubcat)[j]==0) mu2[0]=-mu2[0];			
-				newlogLH=log(normal_cdf(mu2[0]))-0.5*(pow(Ymiss[0]-mumiss[0],2))/omegadrawmiss[0];
-				
-				if ((( double ) unif_rand ( ) )<exp(newlogLH-logLH)) {	
+					mu2[0]=0;
+					for (t=0;t<Il;t++) mu2[0]=mu2[0]+REAL(betaY)[t]*Xsubprop[t];
+					for (t=0;t<Ir;t++) mu2[0]=mu2[0]+REAL(uY)[(INTEGER(clus)[j])+nj*t]*Zsubprop[t];
+					
+					if (REAL(Ysubcat)[j]==0) mu2[0]=-mu2[0];	
+					newlogLH=log(normal_cdf(mu2[0]))-help[0]/2;
+	
+					if ((( double ) unif_rand ( ) )<exp(newlogLH-logLH)) {	
 
+						imp[j+k*IY]=Ymiss[0];
+						help6[0]=help[0];
+						for (t=0;t<Il;t++) Xsub[j+t*IY]=Xsubprop[t];
+						for (t=0;t<Ir;t++) Zsub[j+t*IY]=Zsubprop[t];
+						logLH=newlogLH;
+						accratio=accratio+1;
+					} else {
+						help4[k]=help4[k]-Ymiss[0]+yi[k];
+						yi[k]=imp[j+k*IY];
+					}
+					totprop=totprop+1;				
+				}	else {
+					Ymiss[0]=r8_normal_sample(mumiss[0],sqrt(omegadrawmiss[0]),0);
 					imp[j+k*IY]=Ymiss[0];
-					for (t=0;t<Il;t++) Xsub[j+t*IY]=Xsubprop[t];
-					for (t=0;t<Ir;t++) Zsub[j+t*IY]=Zsubprop[t];
-					logLH=newlogLH;
-					 accratio=accratio+1;
-				} else {
-					yi[k]=imp[j+k*IY];
+					help4[k]=help4[k]+Ymiss[0]-yi[k];
+					r8mat_mm_new(1,JY,JY,help4,invomega,help5);
+					r8mat_mmt_new(1,JY,1,help5,help4,help6);
 				}
-				totprop=totprop+1;				
 				nmiss--;
 				counto=0;
 				countoo=0;
@@ -1195,8 +1232,9 @@ for (i=0;i<ns;i++) {
 					mu2[0]=0;
 					for (t=0;t<Il;t++) mu2[0]=mu2[0]+REAL(betaY)[t]*Xsub[tt+IY*t];
 					for (t=0;t<Ir;t++) mu2[0]=mu2[0]+REAL(uY)[j+nj*t]*Zsub[tt+IY*t];
-					if (REAL(Ysubcat)[tt]==0) mu2[0]=-mu2[0];			
-					logLH=log(normal_cdf(mu2[0]))+logLH;				}				
+					if (REAL(Ysubcat)[tt]==0) mu2[0]=-mu2[0];	
+					logLH=log(normal_cdf(mu2[0]))+logLH;
+				}				
 			}
 		}
 		while (nmiss>0) {
@@ -1233,32 +1271,37 @@ for (i=0;i<ns;i++) {
 				mumiss[0]=mumiss[0]+betamiss[0];
 				r8mat_mm_new(1,(Ju-1),1,omegamo,help4,omegadrawmiss);
 				omegadrawmiss[0]=omegamm[0]-omegadrawmiss[0];
-				Ymiss[0]=r8_normal_sample(yi[k-JY*JZ],sqrt(omegamm[0]/10),0);
-				logLH=0;
 
+				if (((k-JY*JZ)<nconnoaux2)||(((k-JY*JZ)>=ncon2)&((k-JY*JZ)<nconcat2))) {
+					Ymiss[0]=r8_normal_sample(yi[k-JY*JZ],sqrt(omegamm[0]/10),0);
+					logLH=-help6[0]/2;
 				for (tt=0;tt<IY;tt++) {
 					if (INTEGER(clus)[tt]==j) {
 						mu2[0]=0;
 						for (t=0;t<Il;t++) mu2[0]=mu2[0]+REAL(betaY)[t]*Xsub[tt+IY*t];
 						for (t=0;t<Ir;t++) mu2[0]=mu2[0]+REAL(uY)[j+nj*t]*Zsub[tt+IY*t];
-						if (REAL(Ysubcat)[tt]==0) mu2[0]=-mu2[0];			
-						logLH=log(normal_cdf(mu2[0]))-0.5*(pow(yi[k-JY*JZ]-mumiss[0],2))/omegadrawmiss[0]+logLH;
+						if (REAL(Ysubcat)[tt]==0) mu2[0]=-mu2[0];	
+						logLH=log(normal_cdf(mu2[0]))+logLH;
 					}				
 				}
 
 				// Controllare se accettabile
-				yi[k-JY*JZ]=Ymiss[0];					
+					help4[k]= help4[k]+Ymiss[0]-yi[k-JY*JZ];
+					r8mat_mm_new(1,Ju,Ju,help4,invomega2,help5);
+					r8mat_mmt_new(1,Ju,1,help5,help4,help);
+					yi[k-JY*JZ]=Ymiss[0];					
 				
-				
-				if (ncat2>0) {
+				if (ncatnoaux2>0) {
 					h=0;
-					for (jj=0;jj<ncat2;jj++) {
+					for (jj=0;jj<ncatnoaux2;jj++) {
 						maxx=yi[(ncon2+h)];
 						nmaxx=0;
-						for (kk=1;kk<(INTEGER(Y2_numcat)[jj]-1);kk++) {
-							if (yi[(ncon2+h+kk)]>maxx) {
-								maxx=yi[(ncon2+h+kk)];
-								nmaxx=kk;
+						if (INTEGER(Y2_numcat)[jj]>2) {
+							for (kk=1;kk<(INTEGER(Y2_numcat)[jj]-1);kk++) {
+								if (yi[(ncon2+h+kk)]>maxx) {
+									maxx=yi[(ncon2+h+kk)];
+									nmaxx=kk;
+								}
 							}
 						}			
 						if (maxx>0) yi2categorized[jj]=nmaxx;
@@ -1274,15 +1317,17 @@ for (i=0;i<ns;i++) {
 				
 				for (c=0;c<IY;c++) {
 					
-					if (ncat>0) {
+					if (ncatnoaux>0) {
 						h=0;
-						for (jj=0;jj<ncat;jj++) {
+						for (jj=0;jj<ncatnoaux;jj++) {
 							maxx=imp[c+IY*(ncon+h)];
 							nmaxx=0;
+							if (INTEGER(Y_numcat)[jj]>2) {
 							for (kk=1;kk<(INTEGER(Y_numcat)[jj]-1);kk++) {
 								if (imp[c+IY*(ncon+h+kk)]>maxx) {
 									maxx=imp[c+IY*(ncon+h+kk)];
 									nmaxx=kk;
+									}
 								}
 							}			
 							if (maxx>0) yicategorized[jj]=nmaxx;
@@ -1328,20 +1373,19 @@ for (i=0;i<ns;i++) {
 						indic=0;
 					}
 				}
-				newlogLH=0;
-
-				for (tt=0;tt<IY;tt++) {
-					if (INTEGER(clus)[tt]==j) {
-						mu2[0]=0;
-						for (t=0;t<Il;t++) mu2[0]=mu2[0]+REAL(betaY)[t]*Xsubprop[tt+IY*t];
-						for (t=0;t<Ir;t++) mu2[0]=mu2[0]+REAL(uY)[j+nj*t]*Zsub[tt+IY*t];
-						if (REAL(Ysubcat)[tt]==0) mu2[0]=-mu2[0];			
-						newlogLH=log(normal_cdf(mu2[0]))-0.5*(pow(Ymiss[0]-mumiss[0],2))/omegadrawmiss[0]+newlogLH;
-					}				
-				}
-
+					newlogLH=-help[0]/2;
+					for (tt=0;tt<IY;tt++) {
+						if (INTEGER(clus)[tt]==j) {
+							mu2[0]=0;
+							for (t=0;t<Il;t++) mu2[0]=mu2[0]+REAL(betaY)[t]*Xsub[tt+IY*t];
+							for (t=0;t<Ir;t++) mu2[0]=mu2[0]+REAL(uY)[j+nj*t]*Zsub[tt+IY*t];
+							if (REAL(Ysubcat)[tt]==0) mu2[0]=-mu2[0];	
+							newlogLH=log(normal_cdf(mu2[0]))+logLH;
+						}				
+					}
 				if ((( double ) unif_rand ( ) )<exp(newlogLH-logLH)) {	
 					imp2[j+(k-JY*JZ)*Iu]=Ymiss[0];
+					help6[0]=help[0];
 					REAL(u)[j+k*Iu]=Ymiss[0]-betamiss[0];
 					for (tt=0;tt<IY;tt++) {
 						if (INTEGER(clus)[tt]==j) {
@@ -1351,40 +1395,50 @@ for (i=0;i<ns;i++) {
 					logLH=newlogLH;
 					 accratio2=accratio2+1;
 				} else {
+					help4[k]= help4[k]-Ymiss[0]+yi[k-JY*JZ];
 					yi[k-JY*JZ]=imp2[j+(k-JY*JZ)*Iu];
+					
 				}
 				totprop2=totprop2+1;				
-				nmiss--;
-				counto=0;
-				countoo=0;
+
+			}	else {
+					Ymiss[0]=r8_normal_sample(mumiss[0],sqrt(omegadrawmiss[0]),0);
+					imp2[j+(k-JY*JZ)*Iu]=Ymiss[0];
+					REAL(u)[j+k*Iu]=Ymiss[0]-betamiss[0];
+					help4[k]= help4[k]+Ymiss[0]-yi[k-JY*JZ];
+					r8mat_mm_new(1,Ju,Ju,help4,invomega2,help5);
+					r8mat_mmt_new(1,Ju,1,help5,help4,help6);
+				}	
+			nmiss--;
+			counto=0;
+			countoo=0;
 
 			}				
 			k++;				
 		}
 	
 	}
-
-		// Rejection sampling for latent normal outcome
-
-	for (t=0;t<IY;t++) {
-		if (!ISNAN(REAL(Ysub)[t])) {
-			kk=0;
-			flag=0;
-			mu2[0]=0;
-			for (k=0;k<Il;k++) mu2[0]=mu2[0]+REAL(betaY)[k]*Xsub[t+k*IX];
-			for (k=0;k<Ir;k++) mu2[0]=mu2[0]+REAL(uY)[(INTEGER(clus)[t])+nj*k]*Zsub[t+k*IX];
-			while (flag==0&&kk<10000) {
-				yi[0]=r8_normal_sample(mu2[0],sqrt(REAL(varY)[0]),0);
-				if ((REAL(Ysub)[t]==1&&yi[0]>0)||(REAL(Ysub)[t]==0&&yi[0]<0)) {
-					impsub[t]=yi[0];
-					flag=1;
-				} else {
-					kk++;
+	
+			// Rejection sampling for latent normal outcome
+	
+		for (t=0;t<IY;t++) {
+			if (!ISNAN(REAL(Ysub)[t])) {
+				kk=0;
+				flag=0;
+				mu2[0]=0;
+				for (k=0;k<Il;k++) mu2[0]=mu2[0]+REAL(betaY)[k]*Xsub[t+k*IX];
+				for (k=0;k<Ir;k++) mu2[0]=mu2[0]+REAL(uY)[(INTEGER(clus)[t])+nj*k]*Zsub[t+k*IX];
+				while (flag==0&&kk<10000) {
+					yi[0]=r8_normal_sample(mu2[0],sqrt(REAL(varY)[0]),0);
+					if ((REAL(Ysub)[t]==1&&yi[0]>0)||(REAL(Ysub)[t]==0&&yi[0]<0)) {
+						impsub[t]=yi[0];
+						flag=1;
+					} else {
+						kk++;
+					}
 				}
 			}
 		}
-	}
-	
 
 		// Update beta of substantive model
 	
@@ -1412,6 +1466,7 @@ for (i=0;i<ns;i++) {
 	r8mat_mm_new(Il,Il,1,invomega2,sumxy,mu);
 	r8mat_pofac(Il,invomega2,help3,5);
 	r8vec_multinormal_sample(Il, mu,help3, REAL(betaY),newbeta,0);	
+
 	for (t=0;t<Il;t++) REAL(betaYpost)[t+i*Il]=REAL(betaY)[t];
 	
 	// Update random effects of substantive model
@@ -1483,6 +1538,7 @@ for (i=0;i<ns;i++) {
 			}
 		}
 
+
 			//Updating residuals
 
 	for (t=0;t<IY;t++) {
@@ -1495,11 +1551,10 @@ for (i=0;i<ns;i++) {
 			}
 	}
 
-			//Not updating omega (fixed to 1)
+//Not updating omega (fixed to 1)
 
 	REAL(varYpost)[i]=REAL(varY)[0];	
 	
-	// Imputing missing outcomes
 	for (t=0;t<IY;t++) {
 		if (ISNAN(REAL(Ysub)[t])) {
 			mu2[0]=0;
@@ -1513,8 +1568,9 @@ for (i=0;i<ns;i++) {
 	if ((i+1)%fl==0) Rprintf(".");
 }
 if (fl==1) Rprintf("\n");
-if (((double)accratio/((double)totprop))<0.3) Rprintf("Warning: acceptance ratio for level 1 variables imputation = %f. This might be a sign that the chain did not mix well. \n" , ((double)accratio/((double)totprop)));
-if (((double)accratio2/((double)totprop2))<0.3) Rprintf("Warning: acceptance ratio for level 2 variables imputation  = %f. This might be a sign that the chain did not mix well. \n" , ((double)accratio2/((double)totprop2)));
+
+if (((double)accratio/((double)totprop))<0.2) Rprintf("Warning: acceptance ratio for level 1 variables imputation = %f. This might be a sign that the chain did not mix well. \n" , ((double)accratio/((double)totprop)));
+if (((double)accratio2/((double)totprop2))<0.2) Rprintf("Warning: acceptance ratio for level 2 variables imputation  = %f. This might be a sign that the chain did not mix well. \n" , ((double)accratio2/((double)totprop2)));
 
 
 for(i=0;i<IY;i++)  {
@@ -1565,7 +1621,6 @@ if (ncat2>0) {
 	}
 
 }
-	
 PutRNGstate();
 UNPROTECT(59);
 return R_NilValue;
