@@ -9,13 +9,13 @@
 #include<Rinternals.h>
 #include<Rmath.h>
 
-SEXP MCMCjomocox(SEXP Ysub, SEXP submod, SEXP ordersub, SEXP Y, SEXP Yimp, SEXP Yimp2, SEXP Yimpcat, SEXP X, SEXP betaY, SEXP betaYpost, SEXP beta, SEXP betapost, SEXP omega, SEXP omegapost, SEXP nstep, SEXP Sp, SEXP Y_numcat, SEXP num_con, SEXP flagrng){
-int indic=0,i,j,k, IY,JY, IX, JX, Io, Jo, Ib, Jb, ns, nmiss=0,t, countm=0, counto=0, countoo=0, jj, tt, kk, ncon,ncat, pos,flag=0,nmaxx,h=0;
-int fl, currncat, Is, Il=0, JXm,accratio=0, totprop=0;
+SEXP MCMCjomocox(SEXP Ysub, SEXP submod, SEXP ordersub, SEXP Y, SEXP Yimp, SEXP Yimp2, SEXP Yimpcat, SEXP X,SEXP betaY, SEXP betaYpost, SEXP beta, SEXP betapost, SEXP omega, SEXP omegapost, SEXP nstep, SEXP Sp, SEXP Y_numcat, SEXP num_con, SEXP flagrng){
+int indic=0,i,j,k, IY,JY, IX, JX, Io, Jo, Ib, Jb, ns, nmiss=0,t, countm=0, counto=0,countoo=0, jj, tt, kk, ncon,ncat, pos,flag=0,nmaxx,h=0;
+int fl, currncat, Is,  Il=0, JXm, accratio=0, totprop=0, nconnoaux, nconcat, ncatnoaux;
 SEXP RdimY, RdimX, Rdimo, Rdimb, Rdims;
-double *betaX, *Yobs, *Ymiss, *mumiss, *omegadrawmiss, *betamiss, *betaobs, *omegaoo, *omegamo, *omegamm, *invomega, *invomega2, *help, *help2, *help3, *imp, *Xsub, *helpLH;
-double *mu, *mu2, *newbeta, *newomega, *yi, *invomega3, *help4, *help5, *help6, *missing, *fixomega,meanom,sdom, *resid, logLH, newlogLH,detom,  sumbetanew=0;
-double maxx=1,maxim,maxim2, *sumxy, *sumxi, *xi, *incrxx, *incrxy, *Xsubprop,*yicategorized, prova=0, *sumexpbeta, *sumbeta, *sumxexpbeta, *sumx2expbeta, *sumexpbetanew;
+double *betaX, *Yobs, *Ymiss, *mumiss, *omegadrawmiss, *betamiss, *betaobs, *omegaoo, *omegamo, *omegamm, *invomega, *invomega2, *help, *help2, *help3, *imp, *yicategorized, *helpLH;
+double *mu, *mu2, *newbeta, *newomega,  *yi, *invomega3, *help4, *help5, *help6, *missing, *fixomega,meanom,sdom, *resid, logLH, newlogLH,detom, sumbetanew=0;
+double maxx,maxim,maxim2, *sumxy, *sumxi,  *xi,  *incrxx, *incrxy, *Xsub, *Xsubprop,  *sumexpbeta, *sumbeta, *sumxexpbeta, *sumx2expbeta, *sumexpbetanew;
 
 /* Protecting R objects from garbage collection and saving matrices dimensions*/ 
 
@@ -53,11 +53,17 @@ nstep=PROTECT(coerceVector(nstep,INTSXP));
 ns=INTEGER(nstep)[0];
 num_con=PROTECT(coerceVector(num_con,INTSXP));
 ncon=INTEGER(num_con)[0];
+nconnoaux=INTEGER(num_con)[1];
+nconcat=INTEGER(num_con)[2];
+ncatnoaux=INTEGER(num_con)[3];
 flagrng=PROTECT(coerceVector(flagrng,INTSXP));
 fl=INTEGER(flagrng)[0];
 if (REAL(Yimpcat)[0]==(-999)) ncat=0;
 else ncat=length(Y_numcat);
 
+
+
+
 for (i=0;i<XLENGTH(ordersub);i++) {
 	for (j=0;j<INTEGER(ordersub)[i];j++) {
 		maxx=maxx*(INTEGER(submod)[3+4*h]);
@@ -66,12 +72,11 @@ for (i=0;i<XLENGTH(ordersub);i++) {
 	Il=Il+maxx;
 	maxx=1;
 }
-
 JXm=JY;
 if (Il>JY) JXm=Il;
 if (JX>JXm) JXm=JX;
 
-//Allocating memory for C objects in R
+/*Allocating memory for C objects in R*/
 
 help = ( double * ) R_alloc ( (JY*JY) , sizeof ( double ) );
 invomega= (double * ) R_alloc ( (JY*JY) , sizeof ( double ) );
@@ -87,7 +92,7 @@ help3 = ( double * ) R_alloc ( JY * JXm * JY * JXm ,sizeof ( double ) );
 invomega2= (double * ) R_alloc ( JY * JXm * JY * JXm, sizeof ( double ) );
 mu = ( double * ) R_alloc ( JY * JXm, sizeof ( double ) );
 newbeta = ( double * ) R_alloc ( JY*JXm  ,sizeof ( double ) );
-mu2 = ( double * ) R_alloc ( JY * JY,sizeof ( double ) );
+mu2 = ( double * ) R_alloc ( JY * JY, sizeof ( double ) );
 newomega = ( double * ) R_alloc ( JY * JY , sizeof ( double ) );
 betaX=( double * ) R_alloc ( JY, sizeof ( double ) );
 imp=( double * ) R_alloc ( IY * JY,sizeof ( double ) );
@@ -116,7 +121,7 @@ sumxexpbeta = ( double * ) R_alloc ( IY+1 , sizeof ( double ) );
 sumx2expbeta=( double * ) R_alloc (  IY+1,sizeof ( double ) );
 helpLH=( double * ) R_alloc (  4,sizeof ( double ) );
 
-// Some initializations 
+/* Some initializations */
 
 for (j=0; j<IY; j++) {
 	missing[j]=0;
@@ -128,10 +133,8 @@ for (j=0; j<IY; j++) {
 }
 r8mat_copy_new(IY, JY, REAL(Yimp2), imp);
 
-for (i=0;i<Ib*Jb;i++) REAL(betapost)[i]=0;
 for (i=0;i<JY*JY;i++) fixomega[i]=0;
 pos=ncon;
-
 if (ncat>0) {
 	for (i=0;i<ncat;i++) {
 		for (j=0;j<(INTEGER(Y_numcat)[i]-1);j++) {
@@ -144,10 +147,11 @@ if (ncat>0) {
 		pos=pos+INTEGER(Y_numcat)[i]-1;
 	}
 }
+
 for (i=0;i<JY*JY;i++) newomega[i]=REAL(omega)[i];
 
-h=0;
 for (i=0;i<(IY*Il);i++) Xsub[i]=1;
+h=0;
 for (i=0;i<IY;i++) {
 	for (j=0;j<XLENGTH(ordersub);j++) {
 		pos=1;
@@ -180,6 +184,7 @@ for (i=0;i<IY;i++) {
 GetRNGstate();
 
 /* Running ns iterations of Gibbs sampler*/
+
 for (i=0;i<ns;i++) {
 
 	// Rejection Sampling
@@ -187,7 +192,10 @@ for (i=0;i<ns;i++) {
 			pos=ncon;
 			for (j=0;j<ncat;j++) {
 				currncat=INTEGER(Y_numcat)[j]-1;
+							
+
 				for (t=0;t<IY;t++) {
+
 					if (!ISNAN(REAL(Y)[t+(ncon+j)*IY])) {
 						for (k=0;k<currncat;k++) betaX[k]=0;
 						for (tt=0;tt<JX;tt++) {
@@ -195,6 +203,7 @@ for (i=0;i<ns;i++) {
 								betaX[k]=betaX[k]+REAL(beta)[tt+(k+pos)*Ib]*REAL(X)[t+tt*IX];
 							}
 						}
+						
 						for (k=0;k<(JY-currncat);k++) help[k]=0;
 						for (tt=0;tt<JX;tt++) {
 							for (k=0;k<pos;k++) {
@@ -206,7 +215,8 @@ for (i=0;i<ns;i++) {
 						}
 						for (k=0;k<pos;k++) {
 							help[k]=imp[t+k*IY]-help[k];
-						}				
+						}
+				
 						for (k=(pos+currncat);k<JY;k++) {
 							help[k-currncat]=imp[t+k*IY]-help[k-currncat];
 						}
@@ -275,6 +285,7 @@ for (i=0;i<ns;i++) {
 
 		}
 		
+
 	//Updating beta
 
 	r8mat_pofac(JY,REAL(omega), help,3);
@@ -304,14 +315,12 @@ for (i=0;i<ns;i++) {
 	r8mat_mm_new(JY*JX,JY*JX,1,invomega2,sumxy,mu);
 	r8mat_pofac(JY * JX,invomega2,help3,5);
 	r8vec_multinormal_sample(JY*JX, mu,help3, REAL(beta),newbeta,0);
-
 	for (j=0;j<Ib;j++) {
-			for (t=0;t<Jb;t++) {
-				REAL(betapost)[j+Ib*t+i*Ib*Jb]=REAL(beta)[j+Ib*t];
-				}
-		}
-
-		//Updating residuals
+		for (t=0;t<Jb;t++) {
+			REAL(betapost)[j+Ib*t+i*Ib*Jb]=REAL(beta)[j+Ib*t];
+			}
+		}	
+	//Updating residuals
 
 	for (t=0;t<IY;t++) {
 		for (j=0; j<JY; j++) {
@@ -319,7 +328,7 @@ for (i=0;i<ns;i++) {
 			for (k=0;k<JX;k++) resid[t+j*IY]=resid[t+j*IY]-REAL(beta)[k+j*JX]*REAL(X)[t+k*IX];
 		}
 	}
-	
+
 	//Updating omega
 	
 	if (ncat==0) {
@@ -341,8 +350,11 @@ for (i=0;i<ns;i++) {
 		r8mat_poinv(JY, help,invomega);
 		for (jj=1;jj<Io;jj++) for (tt=0;tt<jj;tt++) invomega[jj+Io*tt]=invomega[tt+Io*jj];
 		for(k=0;k<Io;k++)  for(j=0;j<Io;j++)  REAL(omega)[k+Io*j]=invomega[k+Io*j];
-		r8mat_add(JY,JY,REAL(omega),REAL(omegapost));
-	}
+		for (j=0;j<JY;j++) {
+			for (t=0;t<JY;t++) {
+				REAL(omegapost)[j+JY*t+i*JY*JY]=REAL(omega)[j+JY*t];
+				}
+			}	}
 	else {
 		flag=0;
 		r8mat_pofac(JY,newomega,help,12);
@@ -403,9 +415,10 @@ for (i=0;i<ns;i++) {
 		for (j=0;j<JY;j++) {
 			for (t=0;t<JY;t++) {
 				REAL(omegapost)[j+JY*t+i*JY*JY]=REAL(omega)[j+JY*t];
+				}
 			}
-		}
 	}
+		
 
 	//imputing missing values compatibly with substantive model
 	
@@ -439,15 +452,16 @@ for (i=0;i<ns;i++) {
 				for (tt=0;tt<JX;tt++) {
 					xi[t+(tt+t*JX)*JY]=REAL(X)[j+IY*tt];
 				}
-			
 			}
+
 			r8mat_mm_new(JY,JY*JX,1,xi,REAL(beta),help6);
 			r8mat_add(JY,1,help6,betaX);
+			
 			for (t=0;t<JY;t++) help4[t]=yi[t]-betaX[t];
 			r8mat_mm_new(1,JY,JY,help4,invomega,help5);
 			r8mat_mmt_new(1,JY,1,help5,help4,help6);
-			
 			logLH=helpLH[0]-help6[0]/2;
+
 		}
 		while (nmiss>0) {
 			if (ISNAN(REAL(Yimp)[j+k*IY])) {
@@ -479,106 +493,116 @@ for (i=0;i<ns;i++) {
 				mumiss[0]=mumiss[0]+betamiss[0];
 				r8mat_mm_new(1,(JY-1),1,omegamo,help3,omegadrawmiss);
 				omegadrawmiss[0]=omegamm[0]-omegadrawmiss[0];
-				Ymiss[0]=r8_normal_sample(yi[k],sqrt(omegamm[0]),0);
 				
-				logLH=helpLH[0]-0.5*(pow(yi[k]-mumiss[0],2))/omegadrawmiss[0];
-				// Controllare se accettabile
-				yi[k]=Ymiss[0];					
-				
-				if (ncat>0) {
-					h=0;
-					for (jj=0;jj<ncat;jj++) {
-						maxx=yi[(ncon+h)];
-						nmaxx=0;
-						for (kk=1;kk<(INTEGER(Y_numcat)[jj]-1);kk++) {
-							if (yi[(ncon+h+kk)]>maxx) {
-								maxx=yi[(ncon+h+kk)];
-								nmaxx=kk;
-							}
-						}	
-						if (maxx>0) {
-							yicategorized[jj]=nmaxx+1;
-						} else {
-						    yicategorized[jj]=INTEGER(Y_numcat)[jj];
+				if ((k<nconnoaux)||((k>=ncon)&(k<nconcat))) {
+					Ymiss[0]=r8_normal_sample(yi[k],sqrt(omegamm[0]/10),0);
+					logLH=helpLH[0]-help6[0]/2;
+
+					// Controllare se accettabile
+					help4[k]=help4[k]+Ymiss[0]-yi[k];
+					r8mat_mm_new(1,JY,JY,help4,invomega,help5);
+					r8mat_mmt_new(1,JY,1,help5,help4,help);
+					yi[k]=Ymiss[0];								
+					if ((ncatnoaux>0)) {
+						h=0;
+						for (jj=0;jj<(ncatnoaux);jj++) {
+							maxx=yi[(ncon+h)];
+							nmaxx=0;
+							if (INTEGER(Y_numcat)[jj]>2) {
+								for (kk=1;kk<(INTEGER(Y_numcat)[jj]-1);kk++) {
+									if (yi[(ncon+h+kk)]>maxx) {
+										maxx=yi[(ncon+h+kk)];
+										nmaxx=kk;
+									}
+								}
+							}			
+							if (maxx>0) yicategorized[jj]=nmaxx;
+							else yicategorized[jj]=INTEGER(Y_numcat)[jj]-1;
+							h=h+INTEGER(Y_numcat)[jj]-1;
 						}
-						h=h+INTEGER(Y_numcat)[jj]-1;
 					}
-				}
+					
+						//Update Xsubprop
+					h=0;
+					indic=0;
+					for (t=0;t<Il;t++) Xsubprop[t]=1;
 
-				//Update Xsubprop
-				h=0;
-				indic=0;
-				for (t=0;t<Il;t++) Xsubprop[t]=1;
-
-				for (jj=0;jj<XLENGTH(ordersub);jj++) {
-					pos=1;
-					currncat=1;
-					for (t=0;t<INTEGER(ordersub)[jj];t++) {
-						pos=pos*(INTEGER(submod)[3+(h+t)*4]);
-					}
-					for (t=0;t<INTEGER(ordersub)[jj];t++) {
-						if (INTEGER(submod)[1+h*4]==1) {
-							for (tt=0;tt<pos;tt++) {
-								Xsubprop[(tt+indic)]=Xsubprop[(tt+indic)]*pow(yi[(INTEGER(submod)[h*Is]-1)],INTEGER(submod)[2+h*Is]);
+					for (jj=0;jj<XLENGTH(ordersub);jj++) {
+						pos=1;
+						currncat=1;
+						for (t=0;t<INTEGER(ordersub)[jj];t++) {
+							pos=pos*(INTEGER(submod)[3+(h+t)*4]);
+						}
+						for (t=0;t<INTEGER(ordersub)[jj];t++) {
+							if (INTEGER(submod)[1+h*4]==1) {
+								for (tt=0;tt<pos;tt++) {
+									Xsubprop[(tt+indic)]=Xsubprop[(tt+indic)]*pow(yi[(INTEGER(submod)[h*Is]-1)],INTEGER(submod)[2+h*Is]);
 
+								}	
+							} else {
+								for (tt=0;tt<pos;tt++) {
+									kk=(tt*currncat)%INTEGER(submod)[3+h*4]+1;
+									Xsubprop[(tt+indic)]=Xsubprop[(tt+indic)]*(yicategorized[(INTEGER(submod)[h*Is]-1)]==kk);
+								}
 							}	
-						} else {
-							for (tt=0;tt<pos;tt++) {
-								kk=(tt*currncat)%INTEGER(submod)[3+h*4]+2;
-								Xsubprop[(tt+indic)]=Xsubprop[(tt+indic)]*(yicategorized[(INTEGER(submod)[h*Is]-1)]==kk);
-							}
-						}	
-						currncat=currncat*INTEGER(submod)[3+h*4];
-						h=h+1;
+							currncat=currncat*INTEGER(submod)[3+h*4];
+							h=h+1;
+						}
+						currncat=1;
+						indic=indic+pos;
+					}		
+ 			
+ 					helpLH[1]=helpLH[0];
+					sumbeta[j]=0;
+					sumbetanew=0;
+					for (jj=0;jj<Il;jj++) {
+						sumbeta[j]=sumbeta[j]+REAL(betaY)[jj]*Xsub[j+IY*jj];
 					}
-					currncat=1;
-					indic=indic+pos;
-				}
-				
-				helpLH[1]=helpLH[0];
-				sumbeta[j]=0;
-				sumbetanew=0;
-				for (jj=0;jj<Il;jj++) {
-					sumbeta[j]=sumbeta[j]+REAL(betaY)[jj]*Xsub[j+IY*jj];
-				}
-				for (jj=0;jj<Il;jj++) {
-					sumbetanew=sumbetanew+REAL(betaY)[jj]*Xsubprop[jj];
-				}
-				helpLH[2]=exp(sumbeta[j])-exp(sumbetanew);
-				for (t=0; t<(j+1); t++) {
-					sumexpbetanew[t]=sumexpbeta[t]-helpLH[2];
-					if (REAL(Ysub)[t+IY]==1) {
-						helpLH[1]=helpLH[1]+log(sumexpbeta[t]/(sumexpbetanew[t]));
+					for (jj=0;jj<Il;jj++) {
+						sumbetanew=sumbetanew+REAL(betaY)[jj]*Xsubprop[jj];
 					}
-				}
-				if (REAL(Ysub)[j+IY]==1) {
-					helpLH[1]=helpLH[1]-sumbeta[j]+sumbetanew;
-				} 
-				newlogLH=helpLH[1]-0.5*(pow(Ymiss[0]-mumiss[0],2))/omegadrawmiss[0];
-	            if (exp(newlogLH-logLH)<1) prova=prova+exp(newlogLH-logLH);
-	else prova=prova+1;
+					helpLH[2]=exp(sumbeta[j])-exp(sumbetanew);
+					for (t=0; t<(j+1); t++) {
+						sumexpbetanew[t]=sumexpbeta[t]-helpLH[2];
+						if (REAL(Ysub)[t+IY]==1) {
+							helpLH[1]=helpLH[1]+log(sumexpbeta[t]/(sumexpbetanew[t]));
+						}
+					}
+					if (REAL(Ysub)[j+IY]==1) {
+						helpLH[1]=helpLH[1]-sumbeta[j]+sumbetanew;
+					} 
+					newlogLH=helpLH[1]-help[0]/2;
+					if ((( double ) unif_rand ( ) )<exp(newlogLH-logLH)) {	
 
-				if ((( double ) unif_rand ( ) )<exp(newlogLH-logLH)) {	
-
-					imp[j+k*IY]=Ymiss[0];
-					for (t=0;t<Il;t++) Xsub[j+t*IY]=Xsubprop[t]; 
-					logLH=newlogLH;
-					accratio=accratio+1;
-					helpLH[0]=helpLH[1];
-					for (t=0;t<(j+1);t++) sumexpbeta[t]=sumexpbetanew[t];
+						imp[j+k*IY]=Ymiss[0];
+						help6[0]=help[0];
+						for (t=0;t<Il;t++) Xsub[j+t*IY]=Xsubprop[t];
+						logLH=newlogLH;
+						accratio=accratio+1;
+						helpLH[0]=helpLH[1];
+						for (t=0;t<(j+1);t++) sumexpbeta[t]=sumexpbetanew[t];
+					} else {
+						help4[k]=help4[k]-Ymiss[0]+imp[j+k*IY];
+						yi[k]=imp[j+k*IY];
+					}
+					totprop=totprop+1;				
 				} else {
-					yi[k]=imp[j+k*IY];
+					Ymiss[0]=r8_normal_sample(mumiss[0],sqrt(omegadrawmiss[0]),0);
+					imp[j+k*IY]=Ymiss[0];
+					help4[k]=help4[k]+Ymiss[0]-yi[k];
+					r8mat_mm_new(1,JY,JY,help4,invomega,help5);
+					r8mat_mmt_new(1,JY,1,help5,help4,help6);
 				}
-totprop=totprop+1;				
 				nmiss--;
 				counto=0;
 				countoo=0;
 			}				
-			k++;				
+			k++;			
 		}			
 		counto=0;
 		countoo=0;			
 	}
+
 	
 	// Update beta of substantive model
 	
@@ -622,18 +646,17 @@ totprop=totprop+1;
 		}
 	
 	}
-	for (j=0;j<Il;j++) REAL(betaYpost)[j+i*Il]=REAL(betaY)[j];	
-	
-	if ((i+1)%fl==0) Rprintf("Iteration %d completed\n",i+1);
-}
-prova=(prova /(double) totprop);
-if (((double)accratio/((double)totprop))<0.3) Rprintf("Warning: acceptance ratio = %f. This might be a sign that the chain did not mix well. \n" , ((double)accratio/((double)totprop)));
+	for (j=0;j<Il;j++) REAL(betaYpost)[j+i*Il]=REAL(betaY)[j];
 
-for (i=0;i<IY;i++)  {
+	if ((i+1)%fl==0) Rprintf(".");
+}
+	if (fl==1) Rprintf("\n");
+if (((double)accratio/((double)totprop))<0.15) Rprintf("Warning: acceptance ratio = %f. This might be a sign that the chain did not mix well. \n" , ((double)accratio/((double)totprop)));
+for(i=0;i<IY;i++)  {
 	for(j=0;j<JY;j++)  {
 		REAL(Yimp2)[i+IY*j]=imp[i+IY*j];
 	}
-		
+	
 	if (ncat>0) {
 		h=0;
 	    for (j=0;j<ncat;j++) {
@@ -652,9 +675,6 @@ for (i=0;i<IY;i++)  {
 	}	
 }
 
-r8mat_divide(1,Il,ns,REAL(betaYpost));	
-r8mat_divide(Ib,Jb,ns,REAL(betapost));
-r8mat_divide(JY,JY,ns,REAL(omegapost));
 PutRNGstate();
 UNPROTECT(24);
 return R_NilValue;
