@@ -10,8 +10,8 @@
 #include<Rmath.h>
 
 SEXP jomo2lmer(SEXP Ysub, SEXP Ysubimp, SEXP submod, SEXP ordersub, SEXP submodran, SEXP Y, SEXP Yimp, SEXP Yimp2, SEXP Yimpcat, SEXP Y2, SEXP Y2imp, SEXP Y2imp2, SEXP Y2impcat, SEXP X, SEXP X2, SEXP Z, SEXP clus, SEXP betaY, SEXP betaYpost, SEXP beta, SEXP beta2, SEXP u, SEXP uY, SEXP betapost, SEXP upost, SEXP uYpost, SEXP beta2post, SEXP varY, SEXP varYpost, SEXP omega, SEXP omegapost, SEXP covuY, SEXP covuYpost, SEXP covu, SEXP covupost, SEXP nstep, SEXP varYprior, SEXP covuYprior, SEXP Sp, SEXP Sup, SEXP Y_numcat, SEXP Y2_numcat, SEXP num_con, SEXP num_con2, SEXP flagrng){
-int indic=0,i,j,k, IY,JY, IX, JX, Io, Jo, Ib, Jb, ns, nmiss=0,t, countm=0, counto=0,countoo=0, jj, tt, kk, ncon,ncat, pos,flag=0,nmaxx,h=0;
-int Iu, Ju, IZ, JZ, nj,c,fl, currncat, IY2, JY2, IX2,JX2,Ib2, Jb2, ncon2, ncat2, JYm, JXm, Is, Il=0, Ir=0,Jr, JZm, Jum, accratio=0, totprop=0, accratio2=0, totprop2=0;
+int indic=0,i,j,k, IY,JY, IX, JX, Io, Jo, Ib, Jb, ns, nmiss=0,t, countm=0, counto=0,countoo=0, jj, tt, kk, ncon,ncat, pos,flag=0,nmaxx,h=0, nconnoaux, nconcat, ncatnoaux;
+int Iu, Ju, IZ, JZ, nj,c,fl, currncat, IY2, JY2, IX2,JX2,Ib2, Jb2, ncon2, ncat2, JYm, JXm, Is, Il=0, Ir=0,Jr, JZm, Jum, accratio=0, totprop=0, accratio2=0, totprop2=0, nconnoaux2, nconcat2, ncatnoaux2;
 SEXP RdimY, RdimX, Rdimo, Rdimb, RdimZ, Rdimu, RdimY2, RdimX2, Rdimb2, Rdims, Rdimr;
 double *betaX, *Yobs, *Ymiss, *mumiss, *omegadrawmiss, *betamiss, *betaobs, *omegaoo, *omegamo, *omegamm, *invomega, *invomega2, *help, *help2, *help3, *imp,*imp2, *zi, *yicategorized;
 double *sumzy, *incrzz, *incrzy, *mu, *mu2, *newbeta, *newomega, *sumzi, *yi, *invomega3, *help4, *help5, *help6, *missing, *fixomega,meanom,sdom, *resid, logLH, newlogLH,detom;
@@ -96,8 +96,14 @@ nstep=PROTECT(coerceVector(nstep,INTSXP));
 ns=INTEGER(nstep)[0];
 num_con=PROTECT(coerceVector(num_con,INTSXP));
 ncon=INTEGER(num_con)[0];
+nconnoaux=INTEGER(num_con)[1];		
+nconcat=INTEGER(num_con)[2];		
+ncatnoaux=INTEGER(num_con)[3];
 num_con2=PROTECT(coerceVector(num_con2,INTSXP));
 ncon2=INTEGER(num_con2)[0];
+nconnoaux2=INTEGER(num_con2)[1];		
+nconcat2=INTEGER(num_con2)[2];		
+ncatnoaux2=INTEGER(num_con2)[3];
 flagrng=PROTECT(coerceVector(flagrng,INTSXP));
 fl=INTEGER(flagrng)[0];
 if (REAL(Yimpcat)[0]==(-999)) ncat=0;
@@ -310,17 +316,19 @@ for (i=0;i<IY;i++) {
 for (i=0;i<(IY*Ir);i++) Zsub[i]=1;
 for (i=0;i<IY;i++) {
 	pos=0;
-	for (k=0;k<Ir;k++) {
-		if (INTEGER(submodran)[1+pos*3]==1) {
+	for (k=0;k<Jr;k++) {
+		jj=0;
+		if (INTEGER(submodran)[1+k*3]==1) {
 			Zsub[i+IY*pos]=imp[i+IY*(INTEGER(submodran)[k*3]-1)];
 		}
-		else if (INTEGER(submodran)[1+pos*3]==2) {
-			currncat=(INTEGER(submodran)[2+pos*3]);
-			for (jj=1;jj<currncat;jj++) {
-				Zsub[i+IY*(pos+jj-1)]=(REAL(Yimpcat)[i+IY*(INTEGER(submodran)[k*3]-1)]==jj);
+		else if (INTEGER(submodran)[1+k*3]==2) {
+			currncat=(INTEGER(submodran)[2+k*3]);
+			for (jj=0;jj<currncat;jj++) {
+				Zsub[i+IY*(pos+jj)]=(REAL(Yimpcat)[i+IY*(INTEGER(submodran)[k*3]-1)]==(jj+2));
 			}
-		}
-		pos=pos+1;
+		} 
+		pos=pos+1+jj;
+
 	}
 }
 
@@ -931,120 +939,138 @@ for (i=0;i<ns;i++) {
 				mumiss[0]=mumiss[0]+betamiss[0];
 				r8mat_mm_new(1,(JY-1),1,omegamo,help3,omegadrawmiss);
 				omegadrawmiss[0]=omegamm[0]-omegadrawmiss[0];
-				Ymiss[0]=r8_normal_sample(yi[k],sqrt(omegamm[0]),0);
 				
-				mu2[0]=impsub[j];
-				for (t=0;t<Il;t++) mu2[0]=mu2[0]-REAL(betaY)[t]*Xsub[j+IY*t];
-				for (t=0;t<Ir;t++) mu2[0]=mu2[0]-REAL(uY)[(INTEGER(clus)[j])+nj*t]*Zsub[j+IY*t];
+				if ((k<nconnoaux)||((k>=ncon)&(k<nconcat))) {
+					Ymiss[0]=r8_normal_sample(yi[k],sqrt(omegamm[0]/10),0);
+					mu2[0]=impsub[j];
+					for (t=0;t<Il;t++) mu2[0]=mu2[0]-REAL(betaY)[t]*Xsub[j+IY*t];
+					for (t=0;t<Ir;t++) mu2[0]=mu2[0]-REAL(uY)[(INTEGER(clus)[j])+nj*t]*Zsub[j+IY*t];
 
-				logLH=-0.5*mu2[0]*mu2[0]/REAL(varY)[0]-0.5*(pow(yi[k]-mumiss[0],2))/omegadrawmiss[0];;
+					logLH=-0.5*mu2[0]*mu2[0]/REAL(varY)[0]-help6[0]/2;
 
 				// Controllare se accettabile
-				yi[k]=Ymiss[0];								
-				if (ncat>0) {
-					h=0;
-					for (jj=0;jj<ncat;jj++) {
-						maxx=yi[(ncon+h)];
-						nmaxx=0;
-						for (kk=1;kk<(INTEGER(Y_numcat)[jj]-1);kk++) {
-							if (yi[(ncon+h+kk)]>maxx) {
-								maxx=yi[(ncon+h+kk)];
-								nmaxx=kk;
-							}
-						}			
-						if (maxx>0) yicategorized[jj]=nmaxx;
-						else yicategorized[jj]=INTEGER(Y_numcat)[jj]-1;
-						h=h+INTEGER(Y_numcat)[jj]-1;
+					help4[k]=help4[k]+Ymiss[0]-yi[k];
+					r8mat_mm_new(1,JY,JY,help4,invomega,help5);
+					r8mat_mmt_new(1,JY,1,help5,help4,help);
+					yi[k]=Ymiss[0];								
+					if (ncatnoaux>0) {
+						h=0;
+						for (jj=0;jj<ncatnoaux;jj++) {
+							maxx=yi[(ncon+h)];
+							nmaxx=0;
+							if (INTEGER(Y_numcat)[jj]>2) {
+								for (kk=1;kk<(INTEGER(Y_numcat)[jj]-1);kk++) {
+									if (yi[(ncon+h+kk)]>maxx) {
+										maxx=yi[(ncon+h+kk)];
+										nmaxx=kk;
+									}
+								}
+							}			
+							if (maxx>0) yicategorized[jj]=nmaxx;
+							else yicategorized[jj]=INTEGER(Y_numcat)[jj]-1;
+							h=h+INTEGER(Y_numcat)[jj]-1;
+						}
 					}
-				}
-				if (ncat2>0) {
-					h=0;
-					for (jj=0;jj<ncat2;jj++) {
-						maxx=imp2[INTEGER(clus)[j]+Iu*(ncon2+h)];
-						nmaxx=0;
-						for (kk=1;kk<(INTEGER(Y2_numcat)[jj]-1);kk++) {
-							if (imp2[INTEGER(clus)[j]+Iu*(ncon2+h+kk)]>maxx) {
-								maxx=imp2[INTEGER(clus)[j]+Iu*(ncon2+h+kk)];
-								nmaxx=kk;
-							}
-						}			
-						if (maxx>0) yi2categorized[jj]=nmaxx;
-						else yi2categorized[jj]=INTEGER(Y2_numcat)[jj]-1;
-						h=h+INTEGER(Y2_numcat)[jj]-1;
+					if (ncatnoaux2>0) {
+						h=0;
+						for (jj=0;jj<ncatnoaux2;jj++) {
+							maxx=imp2[INTEGER(clus)[j]+Iu*(ncon2+h)];
+							nmaxx=0;
+							if (INTEGER(Y2_numcat)[jj]>2) {
+								for (kk=1;kk<(INTEGER(Y2_numcat)[jj]-1);kk++) {
+									if (imp2[INTEGER(clus)[j]+Iu*(ncon2+h+kk)]>maxx) {
+										maxx=imp2[INTEGER(clus)[j]+Iu*(ncon2+h+kk)];
+										nmaxx=kk;
+									}
+								}
+							}			
+							if (maxx>0) yi2categorized[jj]=nmaxx;
+							else yi2categorized[jj]=INTEGER(Y2_numcat)[jj]-1;
+							h=h+INTEGER(Y2_numcat)[jj]-1;
+						}
 					}
-				}
 					
 					//Update Xsubprop
-				h=0;
-				indic=0;
-				for (t=0;t<Il;t++) Xsubprop[t]=1;
+					h=0;
+					indic=0;
+					for (t=0;t<Il;t++) Xsubprop[t]=1;
 
-				for (jj=0;jj<XLENGTH(ordersub);jj++) {
-					pos=1;
-					currncat=1;
-					for (t=0;t<INTEGER(ordersub)[jj];t++) {
-						pos=pos*(INTEGER(submod)[3+(h+t)*4]);
-					}
-					for (t=0;t<INTEGER(ordersub)[jj];t++) {
-						if (INTEGER(submod)[1+h*4]==1) {
-							for (tt=0;tt<pos;tt++) {
-								Xsubprop[1+tt+indic]=Xsubprop[1+tt+indic]*pow(yi[(INTEGER(submod)[h*Is]-1)],INTEGER(submod)[2+h*Is]);
-
-							}	
-						} else if (INTEGER(submod)[1+h*4]==2) {
-							for (tt=0;tt<pos;tt++) {
-								kk=(tt*currncat)%INTEGER(submod)[3+h*4]+1;
-								Xsubprop[1+tt+indic]=Xsubprop[1+tt+indic]*(yicategorized[(INTEGER(submod)[h*Is]-1)]==kk);
-							}
-						} else if (INTEGER(submod)[1+h*4]==3) {
-							for (tt=0;tt<pos;tt++) {
-								Xsubprop[1+tt+indic]=Xsubprop[1+tt+indic]*pow(imp2[INTEGER(clus)[j]+Iu*(INTEGER(submod)[h*Is]-1)],INTEGER(submod)[2+h*Is]);
-							}
-						} else if (INTEGER(submod)[1+h*4]==4) {
-							for (tt=0;tt<pos;tt++) {
-								kk=(tt*currncat)%INTEGER(submod)[3+h*4]+1;
-								Xsubprop[1+tt+indic]=Xsubprop[1+tt+indic]*(yi2categorized[(INTEGER(submod)[h*Is]-1)]==kk);
-							}
+					for (jj=0;jj<XLENGTH(ordersub);jj++) {
+						pos=1;
+						currncat=1;
+						for (t=0;t<INTEGER(ordersub)[jj];t++) {
+							pos=pos*(INTEGER(submod)[3+(h+t)*4]);
 						}
-						currncat=currncat*INTEGER(submod)[3+h*4];
-						h=h+1;
-					}
-					currncat=1;
-					indic=indic+pos;
-				}
-				
-				   // Update Zsubprop
-
-				for (t=0;t<Ir;t++) Zsubprop[t]=1;
-				
-				for (t=0;t<Ir;t++) {
-					if (INTEGER(submodran)[1+t*3]==1) {
-						Zsubprop[t]=yi[(INTEGER(submodran)[t*3]-1)];
-					} else if (INTEGER(submodran)[1+t*3]==2) {
-						currncat=(INTEGER(submodran)[2+t*3]);
-						for (jj=1;jj<currncat;jj++) {
-							Zsubprop[t+jj-1]=(yicategorized[(INTEGER(submodran)[t*3]-1)]==jj);
+						for (t=0;t<INTEGER(ordersub)[jj];t++) {
+							if (INTEGER(submod)[1+h*4]==1) {
+								for (tt=0;tt<pos;tt++) {
+									Xsubprop[1+tt+indic]=Xsubprop[1+tt+indic]*pow(yi[(INTEGER(submod)[h*Is]-1)],INTEGER(submod)[2+h*Is]);
+								}	
+							} else if (INTEGER(submod)[1+h*4]==2) {
+								for (tt=0;tt<pos;tt++) {
+									kk=(tt*currncat)%INTEGER(submod)[3+h*4]+1;
+									Xsubprop[1+tt+indic]=Xsubprop[1+tt+indic]*(yicategorized[(INTEGER(submod)[h*Is]-1)]==kk);
+								}
+							} else if (INTEGER(submod)[1+h*4]==3) {
+								for (tt=0;tt<pos;tt++) {
+									Xsubprop[1+tt+indic]=Xsubprop[1+tt+indic]*pow(imp2[INTEGER(clus)[j]+Iu*(INTEGER(submod)[h*Is]-1)],INTEGER(submod)[2+h*Is]);
+								}
+							} else if (INTEGER(submod)[1+h*4]==4) {
+								for (tt=0;tt<pos;tt++) {
+									kk=(tt*currncat)%INTEGER(submod)[3+h*4]+1;
+									Xsubprop[1+tt+indic]=Xsubprop[1+tt+indic]*(yi2categorized[(INTEGER(submod)[h*Is]-1)]==kk);
+								}
+							}
+							currncat=currncat*INTEGER(submod)[3+h*4];
+							h=h+1;
 						}
+						currncat=1;
+						indic=indic+pos;
 					}
-				pos=pos+1;
-				}				
 				
-				mu2[0]=impsub[j];
-				for (t=0;t<Il;t++) mu2[0]=mu2[0]-REAL(betaY)[t]*Xsubprop[t];
-				for (t=0;t<Ir;t++) mu2[0]=mu2[0]-REAL(uY)[(INTEGER(clus)[j])+nj*t]*Zsubprop[t];
-				newlogLH=-0.5*mu2[0]*mu2[0]/REAL(varY)[0]-0.5*(pow(Ymiss[0]-mumiss[0],2))/omegadrawmiss[0];
+											// Update Zsubprop
+
+					for (t=0;t<Ir;t++) Zsubprop[t]=1;
+					pos=0;
+					for (t=0;t<Jr;t++) {
+						jj=0;
+						if (INTEGER(submodran)[1+t*3]==1) {
+							Zsubprop[t]=yi[(INTEGER(submodran)[t*3]-1)];
+						} else if (INTEGER(submodran)[1+t*3]==2) {
+							currncat=(INTEGER(submodran)[2+t*3]);
+							for (jj=0;jj<currncat;jj++) {
+								Zsubprop[pos+jj]=(yicategorized[(INTEGER(submodran)[t*3]-1)]==(jj+1));
+							}
+						} 
+						pos=pos+1+jj;
+					}						
+						
+				
+					mu2[0]=impsub[j];
+					for (t=0;t<Il;t++) mu2[0]=mu2[0]-REAL(betaY)[t]*Xsubprop[t];
+					for (t=0;t<Ir;t++) mu2[0]=mu2[0]-REAL(uY)[(INTEGER(clus)[j])+nj*t]*Zsubprop[t];
+					newlogLH=-0.5*mu2[0]*mu2[0]/REAL(varY)[0]-help[0]/2;
 	
-				if ((( double ) unif_rand ( ) )<exp(newlogLH-logLH)) {	
+					if ((( double ) unif_rand ( ) )<exp(newlogLH-logLH)) {	
 
-					imp[j+k*IY]=Ymiss[0];
-					for (t=0;t<Il;t++) Xsub[j+t*IY]=Xsubprop[t];
-					for (t=0;t<Ir;t++) Zsub[j+t*IY]=Zsubprop[t];
-					logLH=newlogLH;
-					 accratio=accratio+1;
+						imp[j+k*IY]=Ymiss[0];
+						help6[0]=help[0];
+						for (t=0;t<Il;t++) Xsub[j+t*IY]=Xsubprop[t];
+						for (t=0;t<Ir;t++) Zsub[j+t*IY]=Zsubprop[t];
+						logLH=newlogLH;
+						accratio=accratio+1;
+					} else {
+						help4[k]=help4[k]-Ymiss[0]+imp[j+k*IY];
+						yi[k]=imp[j+k*IY];
+					}
+					totprop=totprop+1;				
 				} else {
-					yi[k]=imp[j+k*IY];
+					Ymiss[0]=r8_normal_sample(mumiss[0],sqrt(omegadrawmiss[0]),0);
+					imp[j+k*IY]=Ymiss[0];
+					help4[k]=help4[k]+Ymiss[0]-yi[k];
+					r8mat_mm_new(1,JY,JY,help4,invomega,help5);
+					r8mat_mmt_new(1,JY,1,help5,help4,help6);
 				}
-				totprop=totprop+1;				
 				nmiss--;
 				counto=0;
 				countoo=0;
@@ -1125,128 +1151,146 @@ for (i=0;i<ns;i++) {
 				mumiss[0]=mumiss[0]+betamiss[0];
 				r8mat_mm_new(1,(Ju-1),1,omegamo,help4,omegadrawmiss);
 				omegadrawmiss[0]=omegamm[0]-omegadrawmiss[0];
-				Ymiss[0]=r8_normal_sample(yi[k-JY*JZ],sqrt(omegamm[0]/10),0);
-				logLH=0;
 
-				for (tt=0;tt<IY;tt++) {
-					if (INTEGER(clus)[tt]==j) {
-						mu2[0]=impsub[tt];
-						for (t=0;t<Il;t++) mu2[0]=mu2[0]-REAL(betaY)[t]*Xsub[tt+IY*t];
-						for (t=0;t<Ir;t++) mu2[0]=mu2[0]-REAL(uY)[j+nj*t]*Zsub[tt+IY*t];
-						logLH=-0.5*mu2[0]*mu2[0]/REAL(varY)[0]-0.5*(pow(yi[k-JY*JZ]-mumiss[0],2))/omegadrawmiss[0]+logLH;
-					}				
-				}
+				if (((k-JY*JZ)<nconnoaux2)||(((k-JY*JZ)>=ncon2)&((k-JY*JZ)<nconcat2))) {
+					Ymiss[0]=r8_normal_sample(yi[k-JY*JZ],sqrt(omegamm[0]/10),0);
+					logLH=-help6[0]*0.5;
 
-				// Controllare se accettabile
-				yi[k-JY*JZ]=Ymiss[0];					
-				
-				
-				if (ncat2>0) {
-					h=0;
-					for (jj=0;jj<ncat2;jj++) {
-						maxx=yi[(ncon2+h)];
-						nmaxx=0;
-						for (kk=1;kk<(INTEGER(Y2_numcat)[jj]-1);kk++) {
-							if (yi[(ncon2+h+kk)]>maxx) {
-								maxx=yi[(ncon2+h+kk)];
-								nmaxx=kk;
-							}
-						}			
-						if (maxx>0) yi2categorized[jj]=nmaxx;
-						else yi2categorized[jj]=INTEGER(Y2_numcat)[jj]-1;
-						h=h+INTEGER(Y2_numcat)[jj]-1;
-					}
-				}
-				//Update Xsubprop
-				h=0;
-				indic=0;
-				for (t=0;t<Il*IY;t++) Xsubprop[t]=1;
-				
-				for (c=0;c<IY;c++) {
-					
-					if (ncat>0) {
-						h=0;
-						for (jj=0;jj<ncat;jj++) {
-							maxx=imp[c+IY*(ncon+h)];
-							nmaxx=0;
-							for (kk=1;kk<(INTEGER(Y_numcat)[jj]-1);kk++) {
-								if (imp[c+IY*(ncon+h+kk)]>maxx) {
-									maxx=imp[c+IY*(ncon+h+kk)];
-									nmaxx=kk;
-								}
-							}			
-							if (maxx>0) yicategorized[jj]=nmaxx;
-							else yicategorized[jj]=INTEGER(Y_numcat)[jj]-1;
-							h=h+INTEGER(Y_numcat)[jj]-1;
-						}
-					}
-					h=0;
-					if (INTEGER(clus)[c]==j) {
-						for (jj=0;jj<XLENGTH(ordersub);jj++) {
-							pos=1;
-							currncat=1;
-							for (t=0;t<INTEGER(ordersub)[jj];t++) {
-								pos=pos*(INTEGER(submod)[3+(h+t)*4]);
-							}
-							for (t=0;t<INTEGER(ordersub)[jj];t++) {
-								if (INTEGER(submod)[1+h*4]==1) {
-									for (tt=0;tt<pos;tt++) {
-										Xsubprop[c+IY*(1+tt+indic)]=Xsubprop[c+IY*(1+tt+indic)]*pow(imp[c+IY*(INTEGER(submod)[h*Is]-1)],INTEGER(submod)[2+h*Is]);
-									}	
-								} else if (INTEGER(submod)[1+h*4]==2) {
-									for (tt=0;tt<pos;tt++) {
-										kk=(tt*currncat)%INTEGER(submod)[3+h*4]+1;
-										Xsubprop[c+IY*(1+tt+indic)]=Xsubprop[c+IY*(1+tt+indic)]*(yicategorized[(INTEGER(submod)[h*Is]-1)]==kk);
-									}
-								} else if (INTEGER(submod)[1+h*4]==3) {
-									for (tt=0;tt<pos;tt++) {
-										Xsubprop[c+IY*(1+tt+indic)]=Xsubprop[c+IY*(1+tt+indic)]*pow(yi[(INTEGER(submod)[h*Is]-1)],INTEGER(submod)[2+h*Is]);
-									}	
-								} else if (INTEGER(submod)[1+h*4]==4) {
-									for (tt=0;tt<pos;tt++) {
-										kk=(tt*currncat)%INTEGER(submod)[3+h*4]+1;
-										Xsubprop[c+IY*(1+tt+indic)]=Xsubprop[c+IY*(1+tt+indic)]*(yi2categorized[(INTEGER(submod)[h*Is]-1)]==kk);
-									}
-								} 
-								currncat=currncat*INTEGER(submod)[3+h*4];
-								h=h+1;
-							}
-							currncat=1;
-							indic=indic+pos;
-						}
-						h=0;
-						indic=0;
-					}
-				}
-				newlogLH=0;
-
-				for (tt=0;tt<IY;tt++) {
-					if (INTEGER(clus)[tt]==j) {
-						mu2[0]=impsub[tt];
-						for (t=0;t<Il;t++) mu2[0]=mu2[0]-REAL(betaY)[t]*Xsubprop[tt+IY*t];
-						for (t=0;t<Ir;t++) mu2[0]=mu2[0]-REAL(uY)[j+nj*t]*Zsub[tt+IY*t];
-						newlogLH=-0.5*mu2[0]*mu2[0]/REAL(varY)[0]-0.5*(pow(Ymiss[0]-mumiss[0],2))/omegadrawmiss[0]+newlogLH;
-					}				
-				}
-
-				if ((( double ) unif_rand ( ) )<exp(newlogLH-logLH)) {	
-					imp2[j+(k-JY*JZ)*Iu]=Ymiss[0];
-					REAL(u)[j+k*Iu]=Ymiss[0]-betamiss[0];
 					for (tt=0;tt<IY;tt++) {
 						if (INTEGER(clus)[tt]==j) {
-							for (t=0;t<Il;t++) Xsub[tt+t*IY]=Xsubprop[tt+t*IY]; 
+							mu2[0]=impsub[tt];
+							for (t=0;t<Il;t++) mu2[0]=mu2[0]-REAL(betaY)[t]*Xsub[tt+IY*t];
+							for (t=0;t<Ir;t++) mu2[0]=mu2[0]-REAL(uY)[j+nj*t]*Zsub[tt+IY*t];
+							logLH=-0.5*mu2[0]*mu2[0]/REAL(varY)[0]+logLH;
+						}				
+					}
+
+				// Controllare se accettabile
+					help4[k]= help4[k]+Ymiss[0]-yi[k-JY*JZ];
+					r8mat_mm_new(1,Ju,Ju,help4,invomega2,help5);
+					r8mat_mmt_new(1,Ju,1,help5,help4,help);
+					yi[k-JY*JZ]=Ymiss[0];					
+				
+				
+					if (ncatnoaux2>0) {
+						h=0;
+						for (jj=0;jj<ncatnoaux2;jj++) {
+							maxx=yi[(ncon2+h)];
+							nmaxx=0;
+							if (INTEGER(Y2_numcat)[jj]>2) {
+								for (kk=1;kk<(INTEGER(Y2_numcat)[jj]-1);kk++) {
+									if (yi[(ncon2+h+kk)]>maxx) {
+										maxx=yi[(ncon2+h+kk)];
+										nmaxx=kk;
+									}
+								}
+							}			
+							if (maxx>0) yi2categorized[jj]=nmaxx;
+							else yi2categorized[jj]=INTEGER(Y2_numcat)[jj]-1;
+							h=h+INTEGER(Y2_numcat)[jj]-1;
+						}	
+					}
+				//Update Xsubprop
+					h=0;
+					indic=0;
+					for (t=0;t<Il*IY;t++) Xsubprop[t]=1;
+				
+					for (c=0;c<IY;c++) {
+					
+						if (ncatnoaux>0) {
+							h=0;
+							for (jj=0;jj<ncatnoaux;jj++) {
+								maxx=imp[c+IY*(ncon+h)];
+								nmaxx=0;
+								if (INTEGER(Y_numcat)[jj]>2) {
+									for (kk=1;kk<(INTEGER(Y_numcat)[jj]-1);kk++) {
+										if (imp[c+IY*(ncon+h+kk)]>maxx) {
+											maxx=imp[c+IY*(ncon+h+kk)];
+											nmaxx=kk;
+											}
+										}
+									}			
+								if (maxx>0) yicategorized[jj]=nmaxx;
+								else yicategorized[jj]=INTEGER(Y_numcat)[jj]-1;
+								h=h+INTEGER(Y_numcat)[jj]-1;
+							}
+						}
+						h=0;
+						if (INTEGER(clus)[c]==j) {
+							for (jj=0;jj<XLENGTH(ordersub);jj++) {
+								pos=1;
+								currncat=1;
+								for (t=0;t<INTEGER(ordersub)[jj];t++) {
+									pos=pos*(INTEGER(submod)[3+(h+t)*4]);
+								}
+								for (t=0;t<INTEGER(ordersub)[jj];t++) {
+									if (INTEGER(submod)[1+h*4]==1) {
+										for (tt=0;tt<pos;tt++) {
+											Xsubprop[c+IY*(1+tt+indic)]=Xsubprop[c+IY*(1+tt+indic)]*pow(imp[c+IY*(INTEGER(submod)[h*Is]-1)],INTEGER(submod)[2+h*Is]);
+										}	
+									} else if (INTEGER(submod)[1+h*4]==2) {
+										for (tt=0;tt<pos;tt++) {
+											kk=(tt*currncat)%INTEGER(submod)[3+h*4]+1;
+											Xsubprop[c+IY*(1+tt+indic)]=Xsubprop[c+IY*(1+tt+indic)]*(yicategorized[(INTEGER(submod)[h*Is]-1)]==kk);
+										}
+									} else if (INTEGER(submod)[1+h*4]==3) {
+										for (tt=0;tt<pos;tt++) {
+											Xsubprop[c+IY*(1+tt+indic)]=Xsubprop[c+IY*(1+tt+indic)]*pow(yi[(INTEGER(submod)[h*Is]-1)],INTEGER(submod)[2+h*Is]);
+										}	
+									} else if (INTEGER(submod)[1+h*4]==4) {
+										for (tt=0;tt<pos;tt++) {
+											kk=(tt*currncat)%INTEGER(submod)[3+h*4]+1;
+											Xsubprop[c+IY*(1+tt+indic)]=Xsubprop[c+IY*(1+tt+indic)]*(yi2categorized[(INTEGER(submod)[h*Is]-1)]==kk);
+										}
+									} 
+									currncat=currncat*INTEGER(submod)[3+h*4];
+									h=h+1;
+								}
+								currncat=1;
+								indic=indic+pos;
+							}
+							h=0;
+							indic=0;
 						}
 					}
-					logLH=newlogLH;
-					 accratio2=accratio2+1;
-				} else {
-					yi[k-JY*JZ]=imp2[j+(k-JY*JZ)*Iu];
+					newlogLH=-help[0]/2;
+
+					for (tt=0;tt<IY;tt++) {
+						if (INTEGER(clus)[tt]==j) {
+							mu2[0]=impsub[tt];
+							for (t=0;t<Il;t++) mu2[0]=mu2[0]-REAL(betaY)[t]*Xsubprop[tt+IY*t];
+							for (t=0;t<Ir;t++) mu2[0]=mu2[0]-REAL(uY)[j+nj*t]*Zsub[tt+IY*t];
+							newlogLH=-0.5*mu2[0]*mu2[0]/REAL(varY)[0]+newlogLH;
+						}				
+					}
+
+					if ((( double ) unif_rand ( ) )<exp(newlogLH-logLH)) {	
+						imp2[j+(k-JY*JZ)*Iu]=Ymiss[0];
+						help6[0]=help[0];
+						REAL(u)[j+k*Iu]=Ymiss[0]-betamiss[0];
+						for (tt=0;tt<IY;tt++) {
+							if (INTEGER(clus)[tt]==j) {
+								for (t=0;t<Il;t++) Xsub[tt+t*IY]=Xsubprop[tt+t*IY]; 
+							}
+						}
+						logLH=newlogLH;
+						accratio2=accratio2+1;
+					} else {
+						help4[k]= help4[k]-Ymiss[0]+imp2[j+(k-JY*JZ)*Iu];
+						yi[k-JY*JZ]=imp2[j+(k-JY*JZ)*Iu];
+					}
+					totprop2=totprop2+1;				
+				}	else {
+					Ymiss[0]=r8_normal_sample(mumiss[0],sqrt(omegadrawmiss[0]),0);
+					imp2[j+(k-JY*JZ)*Iu]=Ymiss[0];
+					REAL(u)[j+k*Iu]=Ymiss[0]-betamiss[0];
+					help4[k]= help4[k]+Ymiss[0]-yi[k-JY*JZ];
+					r8mat_mm_new(1,Ju,Ju,help4,invomega2,help5);
+					r8mat_mmt_new(1,Ju,1,help5,help4,help6);
 				}
-				totprop2=totprop2+1;				
 				nmiss--;
 				counto=0;
-				countoo=0;
-
+				countoo=0;				
 			}				
 			k++;				
 		}
@@ -1379,8 +1423,8 @@ for (i=0;i<ns;i++) {
 }
 if (fl==1) Rprintf("\n");
 
-if (((double)accratio/((double)totprop))<0.3) Rprintf("Warning: acceptance ratio for level 1 variables imputation = %f. This might be a sign that the chain did not mix well. \n" , ((double)accratio/((double)totprop)));
-if (((double)accratio2/((double)totprop2))<0.3) Rprintf("Warning: acceptance ratio for level 2 variables imputation  = %f. This might be a sign that the chain did not mix well. \n" , ((double)accratio2/((double)totprop2)));
+if (((double)accratio/((double)totprop))<0.15) Rprintf("Warning: acceptance ratio for level 1 variables imputation = %f. This might be a sign that the chain did not mix well. \n" , ((double)accratio/((double)totprop)));
+if (((double)accratio2/((double)totprop2))<0.15) Rprintf("Warning: acceptance ratio for level 2 variables imputation  = %f. This might be a sign that the chain did not mix well. \n" , ((double)accratio2/((double)totprop2)));
 
 for(i=0;i<IY;i++)  {
 	for(j=0;j<JY;j++)  {
